/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 1002 $
 * $Date: 2020-04-10 23:31:24 +0200 (Fri, 10 Apr 2020) $
 */
package at.lcm.bu21.general.dtypes.editor2;

import javax.swing.table.AbstractTableModel;

import at.lcm.bu21.general.dtypes.editor2.TDataModel.TPeriodicModel;

/**
 * * This is a table model for a JTable to view TPeriodic. It based on
 * AbstractTableModel.
 * 
 * @author jkaisinger
 *
 */
public class TPeriodicTableModel extends AbstractTableModel {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/**
	 * Local reference of the tBooleanModel. It allows to manipulate the
	 * TBoolean.
	 */
	private TPeriodicModel tPeriodicModel;

	/** Column header names. */
	private String[] colNames = { "Order:", "A:", "B:", "Mag:", "Phase: [DEG.]" };

	/**
	 * Creates an object
	 * 
	 * @param tPeriodicModel
	 *            The TPeriodicModel
	 */
	public TPeriodicTableModel(TPeriodicModel tPeriodicModel) {
		this.tPeriodicModel = tPeriodicModel;
	}

	/**
	 * Get Column name
	 * 
	 * @param column
	 *            Index of column.
	 */
	@Override
	public String getColumnName(int column) {
		return this.colNames[column];
	}

	@Override
	public int getRowCount() {
		return this.tPeriodicModel.getLength();
	}

	@Override
	public int getColumnCount() {
		return this.colNames.length;
	}

	@Override
	public Class<?> getColumnClass(int column) {
		Class<?> res = null;

		if (column == 0) {
			res = Integer.class;
		} else {
			res = Double.class;
		}
		return res;
	}

	@Override
	public boolean isCellEditable(int row, int column) {
		return true;
	}

	@Override
	public Object getValueAt(int row, int column) {

		int[] orders = this.tPeriodicModel.getUsedOrders();
		Object res = null;

		if (column == 0) {
			res = orders[row];
		} else if (column == 1) {
			res = this.tPeriodicModel.getCoeffA(orders[row]);
		} else if (column == 2) {
			res = this.tPeriodicModel.getCoeffB(orders[row]);
		} else if (column == 3) {
			res = this.tPeriodicModel.getMag(orders[row]);
		} else if (column == 4) {
			res = (this.tPeriodicModel.getPha(orders[row]) * 180) / Math.PI;
		}

		// this.fireTableDataChanged();

		return res;
	}

	@Override
	public void setValueAt(Object value, int row, int column) {
		int[] orders = this.tPeriodicModel.getUsedOrders();
		int order = orders[row];

		if (column == 0) {
			int newOrder = Integer.parseInt(value.toString());
			this.tPeriodicModel.resetOrder(order, newOrder);
		} else if (column == 1) {
			double CoeffA = Double.parseDouble(value.toString());
			this.tPeriodicModel.setCoeffA(order, CoeffA);
		} else if (column == 2) {
			double CoeffB = Double.parseDouble(value.toString());
			this.tPeriodicModel.setCoeffB(order, CoeffB);
		} else if (column == 3) {
			double mag = Double.parseDouble(value.toString());
			this.tPeriodicModel.setMag(order, mag);
		} else if (column == 4) {
			double pha = (Double.parseDouble((value).toString()) * Math.PI) / 180;
			this.tPeriodicModel.setPha(order, pha);
		}
		this.fireTableDataChanged();
	}

}
