/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 1022 $
 * $Date: 2020-06-03 14:13:51 +0200 (Wed, 03 Jun 2020) $
 */
package at.lcm.bu21.general.dtypes.unit;

/**
 * Kind of formula-tokens
 *
 * @author Werner Koppelsttter
 */
public enum TokenKind {
	// error token
	/** Undefined */
	NONE("Undefined"),
	// token-classes
	/** Identifier */
	ELEMENT("Element"),
	/** Number */
	NUMBER("Number"),
	/** One */
	ONE("One"),
	// operators end special-sign
	/** Slash (/) */
	SLASH("Slash"),
	/** Left parenthesis (() */
	LPAR("Left parenthesis"),
	/** Right parenthesis ()) */
	RPAR("Right parenthesis"),
	/** Question mark (?) */
	QUESTION("Question mark"),
	// Additional kinds
	/** Append */
	APPEND("Append"),
	/** Exponent */
	EXPONENT("Exponent"),
	/** End of file */
	EOF("End of unit");

	/** Description */
	private final String desc;

	/**
	 * Create TokenKind with description
	 * 
	 * @param desc
	 *            Description
	 */
	private TokenKind(String desc) {
		this.desc = desc;
	}

	/**
	 * Get description of token
	 * 
	 * @return Description
	 */
	public String getDescription() {
		return this.desc;
	}
}
