/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 997 $
 * $Date: 2020-03-19 13:49:34 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.general.math.autodiff;

/**
 * Summation for multiple summands
 * 
 * @author Gnther Weidenholzer
 * @author Werner Koppelsttter
 */
public class Sum extends Func {

	/**
	 * Constructor summation for multiple summands
	 * 
	 * @param args
	 *            Variable count of summands
	 */
	public Sum(Func... args) {
		super(args);
	}

	@Override
	public double value() throws Exception {
		double val = 0.0;

		for (int k = 0; k < this.getNumArgs(); k++) {
			val += this.getArg(k).value();
		}

		return val;
	}

	/**
	 * Derivation of summation
	 * 
	 * @param var
	 *            Variable for derivation
	 * @return Derivative function
	 * @throws Exception
	 *             Derivative exception
	 */
	@Override
	public Func derivative(Variable var) throws Exception {
		Func[] funcs;

		funcs = new Func[this.getNumArgs()];
		for (int k = 0; k < this.getNumArgs(); k++) {
			funcs[k] = this.getArg(k).derivative(var);
		}

		return new Sum(funcs);
	}

	@Override
	public String toString() {
		String str;

		str = "(";
		try {
			str += this.getArg(0).toString();
		} catch (Exception e) {
			str += "'?'";
		}
		for (int k = 1; k < this.getNumArgs(); k++) {
			try {
				str += "+" + this.getArg(k).toString();
			} catch (Exception e) {
				str += "'?'";
			}
		}
		str += ")";

		return str;
	}

}
