/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 997 $
 * $Date: 2020-03-19 13:49:34 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.general.math.autodiff.wrapper;

import org.apache.commons.math3.analysis.DifferentiableUnivariateFunction;
import org.apache.commons.math3.analysis.UnivariateFunction;

import at.lcm.bu21.general.math.autodiff.Func;
import at.lcm.bu21.general.math.autodiff.Variable;

/**
 * 
 * @author gweidenholzer
 * 
 */
public class DifferentiableFunctionWrapper2 implements DifferentiableUnivariateFunction {

	/** */
	private Func f;

	/** */
	private Variable v;

	/**
	 * @param f
	 * @param v
	 */
	public DifferentiableFunctionWrapper2(Func f, Variable v) {

		this.f = f;
		this.v = v;

	}

	@Override
	public double value(double arg0) {

		double val = Double.NaN;
		this.v.setVal(arg0);
		try {
			val = this.f.value();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return val;

	}

	@Override
	public UnivariateFunction derivative() {

		UnivariateFunction deriv = null;

		try {
			deriv = new DifferentiableFunctionWrapper2(this.f.derivative(this.v), this.v);
		} catch (Exception e) {
		}

		return deriv;
	}

}
