from lxml import etree
import re
import numpy as np
from dtypes import TData
import numbers

class TNumeric(np.matrix, TData):
    """
    TNumeric subclasses np.matrix
    Additionally it it implements the MagOpt TData/XML serialization
    """        
        
    # bypass constructor
    def __new__(cls, mat):
        try:
            mat=np.matrix(mat, np.complex)
        except:
            raise TypeError("TNumeric items must be numeric!")    
        return np.real_if_close(mat).view(cls)


    def __str__(self):
        return TNumeric.__repr__(self)
            
    def __repr__(self):
        return super(TNumeric, self).__repr__().replace('matrix', 'TNumeric')


    def __setitem__(self, tup, val):
        if not isinstance(val, numbers.Number):
            raise TypeError("TNumeric items must be numeric!")
        super(TNumeric, self).__setitem__(tup, val)


    # factory method
    @classmethod
    def fromXML(cls, node):

        s=node.text.strip().lower()
        if s == "[]":
            raise Exception("Empty Numeric is not allowed")
        
        if s.startswith("[") and s.endswith("]"):
            s=s[1:-1]
            
        rows=s.split(";")
        m=len(rows)
        
        items=[]
        
        for r in rows:
            tmp=re.split("[ ,]+", r.strip())
            items.append(tmp);
            if len(tmp) != len(items[0]):
                raise Exception("Numeric dimension mismatch!")
                
        n = len(items[0])
        
        ar=np.empty([m, n], np.complex)
        
        for row in range(0, m):
            for col in range(0, n):
                ar[row, col]=TNumeric.str2num(items[row][col])
        
        ar=np.real_if_close(ar)     #make real if imag is close to 0
        mat=np.matrix(ar)           #convert to matrix
        
        return mat.view(cls)        #view as TNumeric!
      
      
            
    def data2xml(self):
        node=etree.Element("Numeric")
        nrows=int(self.shape[0])
        ncols=int(self.shape[1])        

        s="["
        for r in range(0, nrows):
            for c in range(0, ncols):
                re=self[r,c].real
                im=self[r,c].imag
                s=s+str(re)
                if im != 0.0:
                    if im > 0.0:
                        s=s+"+i*"
                    else:
                        s=s+"-i*"
                    s=s+str(im)
                s=s+" "                    
            s=s[:-1]+";"
        s=s[:-1]+"]"
        node.text=s.replace("Infinity", "Inf")        
        return node

                 
    
    @staticmethod
    def str2num(val):
        
        val=val.strip().lower()
        val=val.replace("Inf", "Infinity")
        val=val.replace("Infinityinity", "Infinity")
        
        index=val.find("i*")
        if index >= 0:
            re = float(val[:index-1])
            im = float(val[index-1] + val[index+2:])
            
        elif val.endswith("i"):
            if val.endswith("*i"):
                val = val[0:-2]
            else:
                val = val[0:-1]
            
            index = val.rfind("+")
            if index >0 and val[index-1] =="e":
                start=index
                index=val.rfind("+", start-1)
                if index == -1:
                    index = val.rfind("-", start-1)
                    
            re=float(val[:index])
            im=float(val[index:])
        else:
            re=float(val)
            im=0.
        
        return re + 1j * im    



    @staticmethod
    def complex2str(re, im):
        s=str(re)
        if im != 0.0:
            if im < 0.0:
                s=s+"+i*"
            else:
                s=s+"-i*"
            s=s+str(im)
        return s
