/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9715 $
 * $Date: 2020-03-19 10:01:15 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.symspace.center.views.tree;

import java.awt.Color;
import java.awt.Component;
import java.awt.Rectangle;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import at.lcm.symspace.center.icons.tree.IconTreeTableView;

/**
 * 
 * @author Markus Priglinger
 * @author Werner Koppelsttter
 */
public class ProTreeTableHeaderIdxViewRenderer
		extends ProTreeTableAbstractHeaderRenderer {

	private static final long serialVersionUID = 19700717201305292L;
	protected JTableHeader _header = null;
	protected JLabel _label = null;
	JButton _button = null;
	protected int _column;
	protected boolean _mousePressed = false;

	private ProTreeTable _table = null;

	public ProTreeTableHeaderIdxViewRenderer(ProTreeTable table, String label) {

		this._table = table;
		this.createComponent();
		this._label.setText(label);
		this.setEnabled(true);
	}

	private void createComponent() {

		this.setLayout(null);
		this.setBorder(BorderFactory.createLineBorder(Color.GREEN));

		this._button = new JButton();
		this._button.setBounds(1, 1, 16, 16);
		this._button.setIcon(new ImageIcon(IconTreeTableView.REMOVE_IDX_VIEW));
		this._button.setToolTipText("removes this column from the table");

		this.add(this._button);

		this._label = new JLabel();
		this._label.setBounds(20, 0, 100, 16);
		this.add(this._label);

		this._table.getTableHeader().addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent evt) {
				int dWidth = 0;
				JTable tableView = ProTreeTableHeaderIdxViewRenderer.this._table;
				TableColumnModel columnModel = tableView.getColumnModel();
				int viewColumn = columnModel.getColumnIndexAtX(evt.getX());
				int column = tableView.convertColumnIndexToModel(viewColumn);

				for (int i = 0; i < viewColumn; i++) {
					TableColumn tc = columnModel.getColumn(i);
					dWidth += tc.getWidth();
				}

				Rectangle columnAlignedBox = ProTreeTableHeaderIdxViewRenderer.this._button
						.getBounds();
				columnAlignedBox.x += dWidth;

				// check if in bounds of button
				if (columnAlignedBox.contains(evt.getPoint())) {

					if ((viewColumn == ProTreeTableHeaderIdxViewRenderer.this._column)
							&& (evt.getClickCount() == 1) && (column != -1)) {

						ProTreeTableHeaderIdxViewRenderer.this._mousePressed = true;
						ProTreeTableHeaderIdxViewRenderer.this._button
								.setIcon(new ImageIcon(
										IconTreeTableView.REMOVE_IDX_VIEW_PRESSED));
					}
				} else {
					ProTreeTableHeaderIdxViewRenderer.this._mousePressed = false;
				}
			}

			@Override
			public void mouseReleased(MouseEvent evt) {
				if (ProTreeTableHeaderIdxViewRenderer.this._mousePressed) {
					ProTreeTableHeaderIdxViewRenderer.this._button.setIcon(
							new ImageIcon(IconTreeTableView.REMOVE_IDX_VIEW));
					ProTreeTableHeaderIdxViewRenderer.this._table
							.getIndexViewManager().removeIndexView(
									ProTreeTableHeaderIdxViewRenderer.this);

					ProTreeTableHeaderIdxViewRenderer.this._mousePressed = false;

				}
			}
		});
	}

	@Override
	public Component getTableCellRendererComponent(JTable table, Object value,
			boolean isSelected, boolean hasFocus, int row, int column) {

		if (table != null) {
			this._header = table.getTableHeader();
			if (this._header != null) {
				this.setForeground(this._header.getForeground());
				this.setBackground(this._header.getBackground());
				this.setFont(this._header.getFont());
			}
		}
		this._column = column;
		return this;
	}
}
