/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 10156 $
 * $Date: 2021-04-10 17:32:56 +0200 (Sat, 10 Apr 2021) $
 */
package at.lcm.symspace.gui.views;

import java.awt.event.ActionListener;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.KeyStroke;

/**
 * Menu-bar for gui
 *
 * @author Werner Koppelsttter
 */
public abstract class MenuBar extends JMenuBar {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/**
	 * Refresh menu
	 */
	public abstract void refresh();

	protected JMenuItem makeMenuItem(String name, URL icon, String actionCommand, int mnemonic, KeyStroke stroke,
			String toolTipText, ActionListener al) {
		JMenuItem item;

		if (icon != null) {
			try {
				item = new JMenuItem(name, new ImageIcon(icon));
			} catch (Exception e) {
				item = new JMenuItem(name);
			}
		} else {
			item = new JMenuItem(name);
		}
		item.setEnabled(false);
		item.setActionCommand(actionCommand);
		if (mnemonic >= 0) {
			item.setMnemonic(mnemonic);
		}
		if (stroke != null) {
			item.setAccelerator(stroke);
		}
		item.setToolTipText(toolTipText);
		item.addActionListener(al);

		return item;
	}

	protected JCheckBoxMenuItem makeCheckBoxMenuItem(String name, URL icon, String actionCommand, int mnemonic,
			KeyStroke stroke, String toolTipText, ActionListener al) {
		JCheckBoxMenuItem item;

		if (icon != null) {
			try {
				item = new JCheckBoxMenuItem(name, new ImageIcon(icon), false);
			} catch (Exception e) {
				item = new JCheckBoxMenuItem(name, null, false);
			}
		} else {
			item = new JCheckBoxMenuItem(name, null, false);

		}
		item.setActionCommand(actionCommand);
		item.setMnemonic(mnemonic);
		if (stroke != null) {
			item.setAccelerator(stroke);
		}
		item.setToolTipText(toolTipText);
		item.addActionListener(al);

		return item;

	}

	protected JRadioButtonMenuItem makeRadioButtonMenuItem(String name, URL icon, String actionCommand, int mnemonic,
			KeyStroke stroke, String toolTipText, ActionListener al) {
		JRadioButtonMenuItem item;

		if (icon != null) {
			try {
				item = new JRadioButtonMenuItem(name, new ImageIcon(icon), false);
			} catch (Exception e) {
				item = new JRadioButtonMenuItem(name, null, false);
			}
		} else {
			item = new JRadioButtonMenuItem(name, null, false);

		}
		item.setActionCommand(actionCommand);
		item.setMnemonic(mnemonic);
		if (stroke != null) {
			item.setAccelerator(stroke);
		}
		item.setToolTipText(toolTipText);
		item.addActionListener(al);

		return item;

	}

}
