/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9713 $
 * $Date: 2020-03-19 09:28:58 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.magopt.structure;

import java.io.Serializable;

import javax.xml.stream.XMLStreamConstants;
import javax.xml.stream.XMLStreamReader;
import javax.xml.stream.XMLStreamWriter;

/**
 * Documentation-file information
 * 
 * @author Werner Koppelsttter
 */
public class DocFile implements Serializable {
	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/** Name to register */
	private String name;

	/** Identifier */
	private String paramName;

	/** File name */
	private String file;

	/** Description */
	private String desc;

	/** Checksum */
	private String checksum;

	/**
	 * Get name of docfile
	 * 
	 * @return Name
	 */
	public String getName() {
		return this.name;
	}

	/**
	 * Set name of docfile
	 * 
	 * @param name
	 *            Name
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * Get name of parameter
	 * 
	 * @return Parameter-name
	 */
	public String getParamName() {
		return this.paramName;
	}

	/**
	 * Set name of parameter
	 * 
	 * @param paramName
	 *            Parameter-name
	 */
	public void setParamName(String paramName) {
		this.paramName = paramName;
	}

	/**
	 * Get path of file
	 * 
	 * @return File-path
	 */
	public String getFile() {
		return this.file;
	}

	/**
	 * Set path of file
	 * 
	 * @param file
	 *            File-path
	 */
	public void setFile(String file) {
		this.file = file;
	}

	/**
	 * Get description of docfile
	 * 
	 * @return Description
	 */
	public String getDescription() {
		return this.desc;
	}

	/**
	 * Set description of docfile
	 * 
	 * @param desc
	 *            Description
	 */
	public void setDescription(String desc) {
		this.desc = desc;
	}

	/**
	 * Get checksum of docfile
	 * 
	 * @return Checksum
	 */
	public String getChecksum() {
		return this.checksum;
	}

	/**
	 * Set checksum of docfile
	 * 
	 * @param checksum
	 *            Checksum
	 */
	public void setChecksum(String checksum) {
		this.checksum = checksum;
	}

	/**
	 * Return string representation of given dataset
	 * 
	 * @return String representation
	 */
	public String toString() {
		StringBuffer str = new StringBuffer();
		str.append("DocFile\n");
		str.append("\n\tName        : ");
		str.append(this.name);
		str.append("\n\tDescription : ");
		str.append(this.desc);
		str.append("\n\tFile        : ");
		str.append(this.file);
		str.append("\n\tParamName   : ");
		str.append(this.paramName);
		str.append("\n\tCheckSum    : ");
		str.append(this.checksum);
		return str.toString();
	}

	/**
	 * Read object from xml-file
	 * 
	 * @param xmlReader
	 *            XMLReader to read xml-file
	 * @throws Exception
	 */
	public void xml2obj(XMLStreamReader xmlReader) throws Exception {
		String element;

		xmlReader.next();
		while (xmlReader.getEventType() == XMLStreamConstants.START_ELEMENT) {
			element = xmlReader.getLocalName();

			if (element.equals("Name")) {
				xmlReader.next();
				this.name = new String();
				while (xmlReader.getEventType() != XMLStreamConstants.END_ELEMENT) {
					this.name += xmlReader.getText();
					xmlReader.next();
				}
				xmlReader.next();
			} else if (element.equals("Description")) {
				xmlReader.next();
				this.desc = new String();
				while (xmlReader.getEventType() != XMLStreamConstants.END_ELEMENT) {
					this.desc += xmlReader.getText();
					xmlReader.next();
				}
				xmlReader.next();
			} else if (element.equals("File")) {
				xmlReader.next();
				this.file = new String();
				while (xmlReader.getEventType() != XMLStreamConstants.END_ELEMENT) {
					this.file += xmlReader.getText();
					xmlReader.next();
				}
				xmlReader.next();
			} else if (element.equals("ParamName")) {
				xmlReader.next();
				this.paramName = new String();
				while (xmlReader.getEventType() != XMLStreamConstants.END_ELEMENT) {
					this.paramName += xmlReader.getText();
					xmlReader.next();
				}
				xmlReader.next();
			} else if (element.equals("CheckSum")) {
				xmlReader.next();
				this.checksum = new String();
				while (xmlReader.getEventType() != XMLStreamConstants.END_ELEMENT) {
					this.checksum += xmlReader.getText();
					xmlReader.next();
				}
				xmlReader.next();
			} else {
				throw new Exception("Unrecognized property: "
						+ xmlReader.getLocalName());
			}
		}
		xmlReader.next();
	}

	/**
	 * Write object to xml-file
	 * 
	 * @param xmlWriter
	 *            XMLWriter to write xml-file
	 * @throws Exception
	 */
	public void obj2xml(XMLStreamWriter xmlWriter) throws Exception {
		xmlWriter.writeStartElement("DocFile");

		// Name
		xmlWriter.writeStartElement("Name");
		if (this.name != null) {
			xmlWriter.writeCharacters(this.name);
		}
		xmlWriter.writeEndElement();

		// Description
		xmlWriter.writeStartElement("Description");
		if (this.desc != null) {
			xmlWriter.writeCharacters(this.desc);
		}
		xmlWriter.writeEndElement();

		// File
		xmlWriter.writeStartElement("File");
		if (this.file != null) {
			xmlWriter.writeCharacters(this.file);
		}
		xmlWriter.writeEndElement();

		// ParamName
		xmlWriter.writeStartElement("ParamName");
		if (this.paramName != null) {
			xmlWriter.writeCharacters(this.paramName);
		}
		xmlWriter.writeEndElement();

		// CheckSum
		xmlWriter.writeStartElement("CheckSum");
		if (this.checksum != null) {
			xmlWriter.writeCharacters(this.checksum);
		}
		xmlWriter.writeEndElement();

		xmlWriter.writeEndElement();
	}

}
