/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9713 $
 * $Date: 2020-03-19 09:28:58 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.magopt.structure.converter;

import at.lcm.bu21.general.utils.XMLNode;

/**
 * Convert to V_0_11.
 * 
 * - Update location of function <br>
 * - Add access field to container <br>
 * - Add enable flag to container
 * 
 * @author Werner Koppelsttter
 */
class Converter_11 {

	/**
	 * Convert the given node
	 * 
	 * @param node
	 *            Data to convert
	 * @return Changed data node
	 */
	static XMLNode convert(XMLNode node) {
		// Convert Parameter
		Converter_11.updateLocation(node);
		Converter_11.addAccess2Container(node);

		return node;
	}

	/**
	 * Update location of functions
	 * 
	 * @param node
	 *            Node to update
	 */
	private static void updateLocation(XMLNode node) {

		if ((node.getName().compareTo("ParamMatlabFunction") == 0)
				|| (node.getName().compareTo("ParamJavaFunction") == 0)) {
			for (XMLNode n : node.getNodes()) {
				if (n.getName().compareTo("ClassLocation") == 0) {
					switch (Integer.parseInt(n.getText())) {
					case 0:
						n.setText("ABSOLUTE");
						break;
					case 1:
						n.setText("PROJECT");
						break;
					case 2:
						n.setText("LIBRARY");
						break;
					}
				}
			}
		}

		// next Node
		for (XMLNode n : node.getNodes()) {
			Converter_11.updateLocation(n);
		}
	}

	/**
	 * Add access-field to container
	 * 
	 * @param node
	 *            Node to update
	 */
	private static void addAccess2Container(XMLNode node) {
		XMLNode item;

		if (node.getName().compareTo("ParamContainer") == 0) {
			// add access
			item = new XMLNode("Access");
			item.setText("PUBLIC");
			node.addNode(item);
			// add enable
			item = new XMLNode("Enable");
			item.setText("true");
			node.addNode(item);
		}

		// next Node
		for (XMLNode n : node.getNodes()) {
			Converter_11.addAccess2Container(n);
		}
	}
}
