/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9713 $
 * $Date: 2020-03-19 09:28:58 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.magopt.structure.converter;

import at.lcm.bu21.general.utils.XMLNode;

/**
 * Convert to V_0_42.
 * 
 * - Change LinkedList exports
 * 
 * @author Werner Koppelsttter
 */
public class Converter_42 {

	/**
	 * Convert the given node
	 * 
	 * @param node
	 *            Data to convert
	 * @return Changed data node
	 */
	static XMLNode convert(XMLNode node) {

		// update functions
		Converter_42.convertLinkedField(node);

		return node;
	}

	/**
	 * Convert functions
	 * 
	 * @param node
	 *            Node to convert
	 */
	private static void convertLinkedField(XMLNode node) {
		XMLNode exp;

		if (node.getName().equals("ParamContainer")) {
			for (XMLNode n : node.getNode(1).getNodes()) {
				Converter_42.convertLinkedField(n);
			}
		} else if (node.getNodeSize() > 0) {
			Converter_42.convertLinkedField(node.getNode(0));
		}

		if (node.getName().equals("ParamLinkedField")) {
			// get Exports
			exp = node.getNode(1).getNode(1);
			// remove Exports
			while (exp.getNodeSize() > 0) {
				exp.rmNode(0);
			}
		}
	}
}
