/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9713 $
 * $Date: 2020-03-19 09:28:58 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.magopt.structure.converter;

import at.lcm.bu21.general.utils.XMLNode;

/**
 * Convert to V_0_5.
 * 
 * - Modify length of flags for constant parameters flags: <br>
 * + Modified <br>
 * + Uncertain <br>
 * + Incomplete <br>
 * + UpdateRequired <br>
 * + SimRunning <br>
 * + Comment
 * 
 * @author Werner Koppelsttter
 */
class Converter_5 {

	/** Number of data-sets */
	private static int len = 1;

	/**
	 * Convert the given node
	 * 
	 * @param node
	 *            Data to convert
	 * @return Changed data node
	 */
	static XMLNode convert(XMLNode node) {

		// find current length
		for (XMLNode n : node.getNodes()) {
			if (n.getName().compareTo("Length") == 0) {
				Converter_5.len = Integer.parseInt(n.getText());
			}
		}

		Converter_5.updateConstFlags(node);
		return node;
	}

	/**
	 * Recursive update of flags in structure
	 * 
	 * @param node
	 *            Node to update
	 */
	private static void updateConstFlags(XMLNode node) {

		// convert depends
		if ((node.getName().compareTo("Modified") == 0)
				|| (node.getName().compareTo("Uncertain") == 0)
				|| (node.getName().compareTo("Incomplete") == 0)
				|| (node.getName().compareTo("UpdateRequiered") == 0)
				|| (node.getName().compareTo("SimRunning") == 0)
				|| (node.getName().compareTo("Comment") == 0)) {
			Converter_5.updateItems(node);
		}

		// next Node
		for (XMLNode n : node.getNodes()) {
			Converter_5.updateConstFlags(n);
		}
	}

	/**
	 * Modify length of flags
	 * 
	 * @param node
	 *            Node to update
	 */
	private static void updateItems(XMLNode node) {
		XMLNode item;
		String text;

		if (node.getNodeSize() < Converter_5.len) {
			text = node.getNode(0).getText();
			while (node.getNodeSize() < Converter_5.len) {
				item = new XMLNode("item");
				item.setText(new String(text));
				node.addNode(item);
			}
		}
	}

}
