/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9713 $
 * $Date: 2020-03-19 09:28:58 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.magopt.structure.converter;

import at.lcm.bu21.general.utils.XMLNode;

/**
 * Convert to V_0_7.
 * 
 * - Reorder structure (eventlog to the end) <br>
 * - Add Error-flag to Parameter <br>
 * - Modify structure of error <br>
 * - Switch from on event-log to one per data-set <br>
 * - Remove exceptions-flag from container
 * 
 * @author Werner Koppelsttter
 */
class Converter_7 {

	/** Number of data-sets */
	private static int len = 1;

	/**
	 * Convert the given node
	 * 
	 * @param node
	 *            Data to convert
	 * @return Changed data node
	 */
	static XMLNode convert(XMLNode node) {

		// get current length
		Converter_7.len = Integer.parseInt(node.getNode(1).getText());

		// modify events
		node.rmNode(0);
		node.addNode(Converter_7.getEventLog());

		// Convert Parameter (error)
		Converter_7.updateParameter(node);

		return node;
	}

	/**
	 * Create new event structure
	 * 
	 * @return node with new event structure
	 */
	private static XMLNode getEventLog() {
		XMLNode eventLog, item;

		eventLog = new XMLNode("EventLog");
		for (int i = 0; i < Converter_7.len; i++) {
			item = new XMLNode("item");
			eventLog.addNode(item);
			// buffer-size
			item.addNode(new XMLNode("BufferSize"));
			item.getNode(0).setText("20");

			// buffer
			item.addNode(new XMLNode("EventBuffer"));
		}
		return eventLog;
	}

	/**
	 * Recursive update of node parameter in structure
	 * 
	 * @param node
	 *            Node to update
	 */
	private static void updateParameter(XMLNode node) {
		XMLNode error, item;
		String value = "false";

		// Add Error to node Parameter
		if (node.getName().compareTo("Parameter") == 0) {

			// search for existing Error-flag
			if (node.getNode(node.getNodeSize() - 1).getName()
					.compareTo("Error") == 0) {
				error = node.getNode(node.getNodeSize() - 1);
				value = error.getNode(0).getText();
			} else {
				error = new XMLNode("Error");
				node.addNode(error);
			}

			// expand to the needed length
			while (error.getNodeSize() < Converter_7.len) {
				item = new XMLNode("item");
				item.setText(value);
				error.addNode(item);
			}
		}

		// Remove exception from ParamContainer
		if (node.getName().compareTo("ParamContainer") == 0) {
			for (int i = node.getNodeSize() - 1; i >= 0; i--) {
				if (node.getNode(i).getName().compareTo("Exception") == 0) {
					node.rmNode(i);
				}
			}
		}

		// next Node
		for (XMLNode n : node.getNodes()) {
			Converter_7.updateParameter(n);
		}
	}
}
