/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9713 $
 * $Date: 2020-03-19 09:28:58 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.magopt.structure.jep;

import java.io.File;
import java.io.PrintStream;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Executors;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import at.lcm.bu21.magopt.utils.MagOptConfig;
import jep.Jep;
import jep.JepConfig;
import jep.JepException;
import jep.PyConfig;
import jep.python.PyModule;
import jep.python.PyObject;

/**
 * Proxy for java embedet python (jep) which used always the same thread
 *
 * @author Werner Koppelsttter
 */
public class JepProxy extends ThreadPoolExecutor {

	private static PyConfig config = null;
	private static boolean init = false;

	private Jep jep = null;

	public static void setInitParams(PyConfig config) {
		JepProxy.config = config;
		JepProxy.init = false;
	}

	public static synchronized Jep configJep(PyConfig config) throws JepException {
		StringBuffer path;
		JepConfig c;
		String sharedModules;
		String[] items;
		Jep jep;

		if (!JepProxy.init) {
			JepProxy.init = true;
			if (config != null) {
				Jep.setInitParams(config);
			}

			// TODO change to newer Version
//			if (config == null) {
//				config = new PyConfig();
//			}
//			try {
//				config.setPythonHome(new File(MagOptConfig.getConfig("PythonPath")));
//			} catch (Throwable e) {
//				System.out.println("Jep: Init PythonPath not possible");
//			}
//			Jep.setInitParams(config);
		}

		// include path
		path = new StringBuffer();

		for (File p : MagOptConfig.getPyPath()) {
			path.append(File.pathSeparator);
			path.append(p.getPath());
		}
		path.deleteCharAt(0);

		// FIXME JepConfig
		c = new JepConfig();
		c.setInteractive(false);
		c.setIncludePath(path.toString());
		// c.setClassLoader(JepProxy.class.getClassLoader());
		c.setClassEnquirer(SyMClassList.getInstance());
		sharedModules = MagOptConfig.getConfig("JepSharedModules");
		if (sharedModules != null) {
			items = sharedModules.split(";");
			for (int k = 0; k < items.length; k++) {
				items[k] = items[k].trim();
			}
			c.addSharedModules(items);
		}
		c.setRedirectOutputStreams(true);

		jep = new Jep(c);
		jep.eval("import sys");
		jep.eval("import os");
		jep.eval("import matplotlib");
		jep.eval("matplotlib.use('Agg')");
		jep.eval("from symspace.connection import eval_fnc_method");
		jep.eval("from symspace.connection import getVariables");

		return jep;

	}

	public JepProxy() throws InterruptedException, ExecutionException {
		super(1, 1, 0L, TimeUnit.MILLISECONDS, new ArrayBlockingQueue<Runnable>(1024), new ThreadFactory() {

			@Override
			public Thread newThread(Runnable r) {
				Thread thread = Executors.defaultThreadFactory().newThread(r);
				thread.setDaemon(true);
				return thread;
			}
		}, new ThreadPoolExecutor.DiscardPolicy());

		this.submit(new Callable<Boolean>() {
			@Override
			public Boolean call() throws JepException {
				JepProxy.this.jep = JepProxy.configJep(JepProxy.config);
				return true;
			}
		}).get();
	}

	public Jep getJep() {
		return this.jep;
	}

	// public void setIncludePath(final String includePath)
	// throws InterruptedException, ExecutionException {
	// this.eval("sys.path += '" + includePath.replace("\\", "\\\\")
	// + "'.split('" + File.pathSeparator + "')");
	// }

	public boolean eval(final String str) throws InterruptedException, ExecutionException {
		return this.submit(new Callable<Boolean>() {
			@Override
			public Boolean call() throws JepException {
				return JepProxy.this.getJep().eval(str);
			}
		}).get();
	}

	public boolean eval(final String... cmd) throws InterruptedException, ExecutionException {
		return this.submit(new Callable<Boolean>() {
			@Override
			public Boolean call() throws JepException {
				boolean b = true;
				int k = 0;

				while (b && (k < cmd.length)) {
					b &= JepProxy.this.getJep().eval(cmd[k]);
					k++;
				}
				return b;
			}
		}).get();
	}

	public Object evalGet(final String var, final String... cmd) throws InterruptedException, ExecutionException {
		return this.submit(new Callable<Object>() {
			@Override
			public Object call() throws JepException {
				boolean b = true;
				int k = 0;

				while (b && (k < cmd.length)) {
					b &= JepProxy.this.getJep().eval(cmd[k]);
					k++;
				}

				if (b) {
					return JepProxy.this.getJep().getValue(var);
				} else {
					return null;
				}
			}
		}).get();

	}

	public Object getValue(final String str) throws InterruptedException, ExecutionException {
		return this.submit(new Callable<Object>() {
			@Override
			public Object call() throws JepException {
				return JepProxy.this.getJep().getValue(str);
			}
		}).get();
	}

	public byte[] getValue_bytearray(final String str) throws InterruptedException, ExecutionException {
		return this.submit(new Callable<byte[]>() {
			@Override
			public byte[] call() throws JepException {
				return JepProxy.this.getJep().getValue_bytearray(str);
			}
		}).get();
	}

	public Object invoke(final String name, final Object... args) throws InterruptedException, ExecutionException {
		return this.submit(new Callable<Object>() {

			@Override
			public Object call() throws JepException {
				return JepProxy.this.getJep().invoke(name, args);
			}
		}).get();
	}

	public void runScript(final String script) throws InterruptedException, ExecutionException {
		this.submit(new Callable<Boolean>() {

			@Override
			public Boolean call() throws JepException {
				JepProxy.this.getJep().runScript(script);
				return true;
			}
		}).get();
	}

	public void runScript(final String script, final ClassLoader cl) throws InterruptedException, ExecutionException {
		this.submit(new Callable<Boolean>() {

			@Override
			public Boolean call() throws JepException {
				JepProxy.this.getJep().runScript(script, cl);
				return true;
			}
		}).get();
	}

	public void set(final String name, final Object v) throws InterruptedException, ExecutionException {
		this.submit(new Callable<Boolean>() {
			@Override
			public Boolean call() throws Exception {
				JepProxy.this.getJep().set(name, v);
				return true;
			}
		}).get();
	}

	public void set(final String name, final String v) throws InterruptedException, ExecutionException {
		this.submit(new Callable<Boolean>() {
			@Override
			public Boolean call() throws Exception {
				JepProxy.this.getJep().set(name, v);
				return true;
			}
		}).get();
	}

	public void set(final String name, final byte v) throws InterruptedException, ExecutionException {
		this.submit(new Callable<Boolean>() {
			@Override
			public Boolean call() throws Exception {
				JepProxy.this.getJep().set(name, v);
				return true;
			}
		}).get();
	}

	public void set(final String name, final byte[] v) throws InterruptedException, ExecutionException {
		this.submit(new Callable<Boolean>() {
			@Override
			public Boolean call() throws Exception {
				JepProxy.this.getJep().set(name, v);
				return true;
			}
		}).get();
	}

	public void set(final String name, final short v) throws InterruptedException, ExecutionException {
		this.submit(new Callable<Boolean>() {
			@Override
			public Boolean call() throws Exception {
				JepProxy.this.getJep().set(name, v);
				return true;
			}
		}).get();
	}

	public void set(final String name, final short[] v) throws InterruptedException, ExecutionException {
		this.submit(new Callable<Boolean>() {
			@Override
			public Boolean call() throws Exception {
				JepProxy.this.getJep().set(name, v);
				return true;
			}
		}).get();
	}

	public void set(final String name, final int v) throws InterruptedException, ExecutionException {
		this.submit(new Callable<Boolean>() {
			@Override
			public Boolean call() throws Exception {
				JepProxy.this.getJep().set(name, v);
				return true;
			}
		}).get();
	}

	public void set(final String name, final int[] v) throws InterruptedException, ExecutionException {
		this.submit(new Callable<Boolean>() {
			@Override
			public Boolean call() throws Exception {
				JepProxy.this.getJep().set(name, v);
				return true;
			}
		}).get();
	}

	public void set(final String name, final long v) throws InterruptedException, ExecutionException {
		this.submit(new Callable<Boolean>() {
			@Override
			public Boolean call() throws Exception {
				JepProxy.this.getJep().set(name, v);
				return true;
			}
		}).get();
	}

	public void set(final String name, final long[] v) throws InterruptedException, ExecutionException {
		this.submit(new Callable<Boolean>() {
			@Override
			public Boolean call() throws Exception {
				JepProxy.this.getJep().set(name, v);
				return true;
			}
		}).get();
	}

	public void set(final String name, final float v) throws InterruptedException, ExecutionException {
		this.submit(new Callable<Boolean>() {
			@Override
			public Boolean call() throws Exception {
				JepProxy.this.getJep().set(name, v);
				return true;
			}
		}).get();
	}

	public void set(final String name, final float[] v) throws InterruptedException, ExecutionException {
		this.submit(new Callable<Boolean>() {
			@Override
			public Boolean call() throws Exception {
				JepProxy.this.getJep().set(name, v);
				return true;
			}
		}).get();
	}

	public void set(final String name, final double v) throws InterruptedException, ExecutionException {
		this.submit(new Callable<Boolean>() {
			@Override
			public Boolean call() throws Exception {
				JepProxy.this.getJep().set(name, v);
				return true;
			}
		}).get();
	}

	public void set(final String name, final double[] v) throws InterruptedException, ExecutionException {
		this.submit(new Callable<Boolean>() {
			@Override
			public Boolean call() throws Exception {
				JepProxy.this.getJep().set(name, v);
				return true;
			}
		}).get();
	}

	public void set(final String name, final char v) throws InterruptedException, ExecutionException {
		this.submit(new Callable<Boolean>() {
			@Override
			public Boolean call() throws Exception {
				JepProxy.this.getJep().set(name, v);
				return true;
			}
		}).get();
	}

	public void set(final String name, final char[] v) throws InterruptedException, ExecutionException {
		this.submit(new Callable<Boolean>() {
			@Override
			public Boolean call() throws Exception {
				JepProxy.this.getJep().set(name, v);
				return true;
			}
		}).get();
	}

	public void set(final String name, final boolean v) throws InterruptedException, ExecutionException {
		this.submit(new Callable<Boolean>() {
			@Override
			public Boolean call() throws Exception {
				JepProxy.this.getJep().set(name, v);
				return true;
			}
		}).get();
	}

	public void set(final String name, final boolean[] v) throws InterruptedException, ExecutionException {
		this.submit(new Callable<Boolean>() {
			@Override
			public Boolean call() throws Exception {
				JepProxy.this.getJep().set(name, v);
				return true;
			}
		}).get();
	}

	public PyModule createModule(final String name) throws InterruptedException, ExecutionException {
		return this.submit(new Callable<PyModule>() {
			@Override
			public PyModule call() throws Exception {
				return JepProxy.this.getJep().createModule(name);
			}
		}).get();
	}

	public boolean isInteractive() throws InterruptedException, ExecutionException {
		return this.submit(new Callable<Boolean>() {
			@Override
			public Boolean call() throws Exception {
				return JepProxy.this.getJep().isInteractive();
			}
		}).get();
	}

	public void isValidThread() throws InterruptedException, ExecutionException {
		this.submit(new Callable<Boolean>() {
			@Override
			public Boolean call() throws Exception {
				JepProxy.this.getJep().isValidThread();
				return true;
			}
		}).get();
	}

	public void setClassLoader(final ClassLoader cl) throws InterruptedException, ExecutionException {
		this.submit(new Callable<Boolean>() {
			@Override
			public Boolean call() throws Exception {
				JepProxy.this.getJep().setClassLoader(cl);
				return true;
			}
		}).get();
	}

	public void setInteractive(final boolean v) throws InterruptedException, ExecutionException {
		this.submit(new Callable<Boolean>() {
			@Override
			public Boolean call() throws Exception {
				JepProxy.this.getJep().setInteractive(v);
				return true;
			}
		}).get();
	}

	public PyObject trackObject(final PyObject obj) throws InterruptedException, ExecutionException {
		return this.submit(new Callable<PyObject>() {
			@Override
			public PyObject call() throws Exception {
				return JepProxy.this.getJep().trackObject(obj);
			}
		}).get();
	}

	public PyObject trackObject(final PyObject obj, final boolean inc) throws InterruptedException, ExecutionException {
		return this.submit(new Callable<PyObject>() {
			@Override
			public PyObject call() throws Exception {
				return JepProxy.this.getJep().trackObject(obj, inc);
			}
		}).get();
	}

	public void close() throws InterruptedException, ExecutionException {
		this.submit(new Callable<Boolean>() {

			@Override
			public Boolean call() throws JepException {
				JepProxy.this.getJep().close();
				return true;
			}
		}).get();
		this.shutdown();
	}

	public void redirectStream(final PrintStream out, final PrintStream err)
			throws InterruptedException, ExecutionException {
		// this.redirectStream(out, Charset.defaultCharset().name(), err,
		// Charset.defaultCharset().name());
		this.redirectStream(out, "cp1252", err, "cp1252");
	}

	public void redirectStream(final PrintStream out, String outEncoding, final PrintStream err, String errEncoding)
			throws InterruptedException, ExecutionException {
		this.eval("from symspace.connection import redirect_output");
		this.invoke("redirect_output", out, outEncoding, err, errEncoding);
	}
}
