/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9713 $
 * $Date: 2020-03-19 09:28:58 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.magopt.structure.rmi;

import java.io.File;
import java.rmi.RemoteException;

import at.lcm.bu21.magopt.structure.IParamDirectory;
import at.lcm.bu21.magopt.structure.Location;
import at.lcm.bu21.magopt.structure.ParamDirectory;
import at.lcm.bu21.magopt.structure.client.CParamDirectory;

/**
 * RMI adapter for parameter
 * 
 * @author Werner Koppelsttter
 */
public class RParamDirectory extends RParameter implements IParamDirectory {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/** Directory class */
	private ParamDirectory param;

	/**
	 * Create rmi adapter for directory
	 * 
	 * @param param
	 *            Directory
	 * @throws RemoteException
	 */
	public RParamDirectory(ParamDirectory param) throws RemoteException {
		super(param);
		this.param = param;
	}

	@Override
	public Location getLocation() {
		return this.param.getLocation();
	}

	@Override
	public void changeLocation(Location location) {
		this.param.changeLocation(location);
	}

	@Override
	public String getRelDirPath() {
		return this.param.getRelDirPath();
	}

	@Override
	public void setRelDirPath(String relPath) {
		this.param.setRelDirPath(relPath);
	}

	@Override
	public File getAbsoluteDirectory() {
		return this.param.getAbsoluteDirectory();
	}

	@Override
	public void setAbsoluteDirectory(File dir) {
		this.param.setAbsoluteDirectory(dir);
	}

	@Override
	public void show() throws RemoteException {
		this.param.show();
	}

	@Override
	public CParamDirectory getClientImpl() {
		return new CParamDirectory(this);
	}

}
