/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9713 $
 * $Date: 2020-03-19 09:28:58 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.magopt.structure.rmi;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

import at.lcm.bu21.magopt.structure.IReportParameter;
import at.lcm.bu21.magopt.structure.ReportParameter;
import at.lcm.bu21.magopt.structure.ReportParameter.Language;
import at.lcm.bu21.magopt.structure.ReportParameter.ReportProperty;
import at.lcm.bu21.magopt.structure.client.CReportParameter;

/**
 * RMI adapter for report-parameter
 * 
 * @author Werner Koppelsttter
 */
public class RReportParameter extends UnicastRemoteObject implements
		IReportParameter {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/** Report class */
	private ReportParameter report;

	/**
	 * Create rmi adapter for report-parameter
	 * 
	 * @param report
	 *            Report parameter
	 * @throws RemoteException
	 */
	public RReportParameter(ReportParameter report) throws RemoteException {
		this.report = report;
	}

	@Override
	public Language getLanguage() {
		return this.report.getLanguage();
	}

	@Override
	public void setLanguage(Language lang) {
		this.report.setLanguage(lang);
	}

	@Override
	public String getStylePath() {
		return this.report.getStylePath();
	}

	@Override
	public String getStyle() {
		return this.report.getStyle();
	}

	@Override
	public void setStyle(String style) {
		this.report.setStyle(style);
	}

	@Override
	public ReportProperty[] getProperties() {
		return this.report.getProperties();
	}

	@Override
	public ReportProperty getProperty(String key) {
		return this.report.getProperty(key);
	}

	@Override
	public void setProperty(String key, String value) {
		this.report.setProperty(key, value);
	}

	@Override
	public CReportParameter getClientImpl() {
		return new CReportParameter(this);
	}
}
