/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9713 $
 * $Date: 2020-03-19 09:28:58 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.magopt.structure.simulate;

import java.util.EventObject;

/**
 * Simulation event
 *
 * @author Werner Koppelsttter
 */
public class SimulateEvent extends EventObject {

	/**
	 * Identifies the type of event
	 *
	 * @author Werner Koppelsttter
	 */
	public enum EventType {
		/** Run simulation flag changed */
		RUN_SIM_CHANGED,
		/** Remove compute flag changed */
		REMOTE_COMPUTE_CHANGED,
		/** Optimize flag changed */
		OPTIMIZE_CHANGED,
		/** Timeout changed */
		TIMEOUT_CHANGED,
		/** Status changed */
		STATUS_CHANGED,
		/** Start simulation */
		START,
		/** Simulation started */
		STARTED,
		/** Stop simulation */
		STOP,
		/** Simulation stopped */
		STOPPED
	}

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/** Type of event */
	private EventType type;

	/**
	 * Create simulation event
	 * 
	 * @param source
	 *            Source object
	 * @param type
	 *            Event-type
	 */
	public SimulateEvent(Object source, EventType type) {
		super(source);
		this.type = type;
	}

	/**
	 * Get type of event
	 * 
	 * @return Event type
	 */
	public EventType getType() {
		return this.type;
	}

}
