/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.gui;

import java.io.File;

import javax.swing.JFileChooser;

import magopt.gui.helpers.JProgressDialog;
import magopt.gui.helpers.JProjectChooser;
import at.lcm.bu21.magopt.structure.Project;

/**
 * Runs Save Process in a non-EDT Thread.
 * 
 * @author Aigner Markus
 * 
 */
public class SaveThread extends BaseThread {
	private File mFile;

	protected SaveThread(MagOptGui gui, boolean overwrite) {
		super(gui);
		boolean doSave;
		if (overwrite == false) {
			JProjectChooser chooser = new JProjectChooser(JProjectChooser.ALL,
					new File("."));

			int returnVal = chooser.showSaveDialog(this.mGui);
			if (returnVal == JFileChooser.APPROVE_OPTION) {
				this.mFile = chooser.getSelectedFile();
				doSave = true;
			} else {
				doSave = false;
			}
		} else {
			doSave = true;
		}

		if (doSave == false) {
			this.cancel(true);
		} else {
			this.mPD = new JProgressDialog(gui, "Save Project", "Saving ...");
		}
	}

	@Override
	protected Void doInBackground() throws Exception {
		if (this.mFile == null) {
			this.mGui.mProject.save();
		} else {
			this.mGui.mProject.saveAs(this.mFile);
		}
		return null;
	}

	@Override
	protected void done() {
		if (this.isCancelled() == true) {
			return;
		}
		this.mGui.setTitle(MagOptGui.GUITITLE + " - " + Project.getVersion()
				+ " - " + this.mGui.mProject.getMopFile().getAbsolutePath());

		this.mGui.updateTreeComponent();
		this.mGui.updateViewComponent();
		this.mGui.updateIndexComponent();
		this.mGui.updateEventComponent();
		this.mPD.setVisible(false);
	}
}
