/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JToolBar;

import at.lcm.bu21.magopt.structure.IParameter;
import at.lcm.bu21.magopt.structure.IProject;

/**
 * Creates Tool Bar for MagOpt UI.
 * 
 * @author Aigner Markus
 * 
 */
public class ToolBar extends JToolBar {
	private static final long serialVersionUID = 1L;

	private static final String ICONDIR = "/magopt/gui/icons/";

	private MagOptGui mGui;
	private JButton mNew, mOpen, mSave;
	private JButton mCut, mCopy, mPaste;
	private JButton mDuplicate, mReduce;
	private JButton mRefresh, mCheck, mRegenerate;

	protected ToolBar(MagOptGui gui) {
		this.mGui = gui;

		this.setFloatable(false);
		ImageIcon iNew = new ImageIcon(
				ToolBar.class.getResource(ToolBar.ICONDIR + "New.png"));
		ImageIcon iOpen = new ImageIcon(
				ToolBar.class.getResource(ToolBar.ICONDIR + "Open.png"));
		ImageIcon iSave = new ImageIcon(
				ToolBar.class.getResource(ToolBar.ICONDIR + "Save.png"));
		ImageIcon iCut = new ImageIcon(
				ToolBar.class.getResource(ToolBar.ICONDIR + "Cut.png"));
		ImageIcon iCopy = new ImageIcon(
				ToolBar.class.getResource(ToolBar.ICONDIR + "Copy.png"));
		ImageIcon iPaste = new ImageIcon(
				ToolBar.class.getResource(ToolBar.ICONDIR + "Paste.png"));
		ImageIcon iDuplicate = new ImageIcon(
				ToolBar.class.getResource(ToolBar.ICONDIR + "Duplicate.png"));
		ImageIcon iReduce = new ImageIcon(
				ToolBar.class.getResource(ToolBar.ICONDIR + "Reduce.png"));
		ImageIcon iCheck = new ImageIcon(
				ToolBar.class.getResource(ToolBar.ICONDIR + "Check.png"));
		ImageIcon iRegenerate = new ImageIcon(
				ToolBar.class.getResource(ToolBar.ICONDIR + "Regenerate.png"));
		ImageIcon iRefresh = new ImageIcon(
				ToolBar.class.getResource(ToolBar.ICONDIR + "Refresh.png"));

		/* new project */
		this.mNew = new JButton(iNew);
		this.mNew.setToolTipText("New Project");
		this.mNew.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				ToolBar.this.mGui.newProject();
			}
		});
		this.add(this.mNew);

		/* open project */
		this.mOpen = new JButton(iOpen);
		this.mOpen.setToolTipText("Open Project");
		this.mOpen.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				ToolBar.this.mGui.openProject();
			}
		});
		this.add(this.mOpen);

		/* save project */
		this.mSave = new JButton(iSave);
		this.mSave.setToolTipText("Save Project");
		this.mSave.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				ToolBar.this.mGui.saveProject(true);
			}
		});
		this.add(this.mSave);

		/* cut parameter */
		this.addSeparator();
		this.addSeparator();
		this.mCut = new JButton(iCut);
		this.mCut.setToolTipText("Cut");
		this.mCut.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				ToolBar.this.mGui.copyParameter(true);
			}
		});
		this.add(this.mCut);

		/* copy parameter */
		this.mCopy = new JButton(iCopy);
		this.mCopy.setToolTipText("Copy");
		this.mCopy.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				ToolBar.this.mGui.copyParameter(false);
			}
		});
		this.add(this.mCopy);

		/* paste parameter */
		this.mPaste = new JButton(iPaste);
		this.mPaste.setToolTipText("Paste");
		this.mPaste.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				ToolBar.this.mGui.pasteParameter();
			}
		});
		this.add(this.mPaste);

		/* add config */
		this.addSeparator();
		this.mDuplicate = new JButton(iDuplicate);
		this.mDuplicate.setToolTipText("Duplicate Data Set");
		this.mDuplicate.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				ToolBar.this.mGui.addDataSet();
			}
		});
		this.add(this.mDuplicate);

		/* remove config */
		this.mReduce = new JButton(iReduce);
		this.mReduce.setToolTipText("Remove Data Set");
		this.mReduce.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				ToolBar.this.mGui.removeDataSet();
			}
		});
		this.add(this.mReduce);

		this.addSeparator();

		/* refresh */
		this.mRefresh = new JButton(iRefresh);
		this.mRefresh.setToolTipText("Refresh");
		this.mRefresh.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				ToolBar.this.mGui.refresh();
			}
		});
		this.add(this.mRefresh);

		/* check */
		this.mCheck = new JButton(iCheck);
		this.mCheck.setToolTipText("Check");
		this.mCheck.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				ToolBar.this.mGui.check();
			}
		});
		this.add(this.mCheck);

		/* regenerate */
		this.mRegenerate = new JButton(iRegenerate);
		this.mRegenerate.setToolTipText("Regenerate All Data Sets");
		this.mRegenerate.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				ToolBar.this.mGui.regenerate(true);
			}
		});
		this.add(this.mRegenerate);

		this.check();
	}

	/**
	 * Enable/Disable Tool Bar Elements.
	 */
	protected void check() {
		if (this.mGui.mProject == null) {
			this.mSave.setEnabled(false);
			this.mCut.setEnabled(false);
			this.mCopy.setEnabled(false);
			this.mPaste.setEnabled(false);
			this.mDuplicate.setEnabled(false);
			this.mReduce.setEnabled(false);
			this.mRefresh.setEnabled(false);
			this.mCheck.setEnabled(false);
			this.mRegenerate.setEnabled(false);
		} else {
			this.mSave.setEnabled(true);

			/*
			 * disables toolbar items 'delete' and 'copy' if selected item is
			 * project
			 */
			IParameter selected = this.mGui.mProjectTree.getSelectedParameter();
			if (selected instanceof IProject) {
				this.mCopy.setEnabled(false);
				this.mCut.setEnabled(false);
			} else {
				this.mCopy.setEnabled(true);
				this.mCut.setEnabled(true);
			}
			this.mPaste.setEnabled(true);
			this.mDuplicate.setEnabled(true);
			this.mReduce.setEnabled(true);
			this.mRefresh.setEnabled(true);
			this.mCheck.setEnabled(true);
			this.mRegenerate.setEnabled(true);
		}
	}
}
