/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.gui.compare;

import java.awt.BorderLayout;

import javax.swing.BoxLayout;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.WindowConstants;
import javax.swing.border.EmptyBorder;

import at.lcm.bu21.magopt.structure.compare.CompareParameter;

/**
 * Display the result of the compare
 * 
 * @author Werner Koppelsttter
 */
public class CompareDialog extends JDialog {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/** JTable to display compare-result */
	protected JTable table;

	/** Compare-model */
	protected CompareModel model;

	/** Field for source 1 */
	protected JTextField source1;

	/** Field for source 2 */
	protected JTextField source2;

	/**
	 * Create compare dialog with results
	 * 
	 * @param compare
	 *            Compare object
	 */
	public CompareDialog(CompareParameter compare) {
		super();

		this.source1 = new JTextField(compare.getSource1());
		this.source1.setEditable(false);
		this.source2 = new JTextField(compare.getSource2());
		this.source2.setEditable(false);
		this.model = new CompareModel();
		this.model.setCompareResult(compare.getResult());

		this.setTitle("Compare result");
		this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		this.setLayout(new BorderLayout());

		JPanel panel = new JPanel();
		panel.setBorder(new EmptyBorder(5, 5, 5, 5));
		panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
		this.add(panel, BorderLayout.NORTH);

		// add descriptor
		JPanel panelName = new JPanel();
		panelName.setLayout(new BoxLayout(panelName, BoxLayout.Y_AXIS));
		panelName.add(new JLabel("Source 1: "));
		panelName.add(new JLabel("Source 2:"));
		panel.add(panelName);

		// add values
		JPanel panelValues = new JPanel();
		panelValues.setLayout(new BoxLayout(panelValues, BoxLayout.Y_AXIS));
		panelValues.add(this.source1);
		panelValues.add(this.source2);
		panel.add(panelValues);

		// add compare-list
		this.table = new JTable();
		this.table.getTableHeader().setReorderingAllowed(false);
		this.table.setModel(this.model);

		// add scroll-pane
		JScrollPane pane = new JScrollPane(this.table);
		pane.setBorder(new EmptyBorder(5, 5, 5, 5));
		this.add(pane, BorderLayout.CENTER);

		// set max width for checkbox column
		this.table.getColumnModel()
				.getColumn(CompareColumn.IDENTIFIER.getValue())
				.setPreferredWidth(350);
		this.table.getColumnModel().getColumn(CompareColumn.REASON.getValue())
				.setPreferredWidth(150);
		this.table.getColumnModel()
				.getColumn(CompareColumn.DIFFERENT.getValue())
				.setPreferredWidth(500);

		this.setSize(1000, 600);
		this.setLocationByPlatform(true);
		this.setVisible(true);

	}
}
