/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.gui.helpers;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;

/**
 * Select user
 * 
 * @author Werner Koppelsttter
 */
public class SelectUserDialog {

	/** Dialog to select user */
	JDialog dialog;

	/** Textfield for user-name */
	protected JTextField userField;
	/** Textfield for password */
	protected JPasswordField passwordField;

	/** Selected user */
	private String user;
	/** Password to selected user */
	private String pwd = "";

	/**
	 * Open dialog
	 */
	public void show() {
		this.dialog.setVisible(true);
	}

	/**
	 * Constructor of dialog to select material
	 * 
	 * @param parent
	 *            Parent of dialog
	 * @param user
	 *            User-name
	 */
	public SelectUserDialog(Frame parent, String user) {
		this.user = user;

		this.init(parent);
		this.dialog.setLocationRelativeTo(parent);
		this.dialog.setSize(new Dimension(300, 150));

	}

	/**
	 * Initialize dialog
	 * 
	 * @param parent
	 *            Parent frame
	 */
	protected void init(Frame parent) {
		JPanel mainPanel, panel;
		JButton button;
		JLabel label;
		GridBagLayout layout;
		GridBagConstraints c;
		// Font font;

		// create dialog
		this.dialog = new JDialog(parent, "Select material", true);
		this.dialog.addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
				SelectUserDialog.this.dialog.setVisible(false);
				SelectUserDialog.this.dialog.dispose();
			}
		});
		this.dialog.setLayout(new BorderLayout());

		mainPanel = new JPanel();
		mainPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		mainPanel.setLayout(new BorderLayout());
		this.dialog.add(mainPanel, BorderLayout.CENTER);

		// // top area
		// label = new JLabel("Selct material");
		// font = label.getFont();
		// font = font.deriveFont(Font.BOLD, font.getSize() * 1.4f);
		// label.setFont(font);
		// mainPanel.add(label, BorderLayout.NORTH);

		// center area
		panel = new JPanel();
		layout = new GridBagLayout();
		c = new GridBagConstraints();
		c.fill = GridBagConstraints.BOTH;
		c.insets = new Insets(5, 5, 5, 5);
		panel.setLayout(layout);
		mainPanel.add(panel, BorderLayout.CENTER);

		label = new JLabel("User-name:");
		c.gridwidth = 1;
		c.weightx = 0.0;
		layout.setConstraints(label, c);
		panel.add(label);
		this.userField = new JTextField();
		this.userField.setText(this.user);
		c.gridwidth = GridBagConstraints.REMAINDER;
		c.weightx = 1.0;
		layout.setConstraints(this.userField, c);
		panel.add(this.userField);

		label = new JLabel("Password:");
		c.gridwidth = 1;
		c.weightx = 0.0;
		layout.setConstraints(label, c);
		panel.add(label);
		this.passwordField = new JPasswordField();
		this.passwordField.setText(this.pwd);
		c.gridwidth = GridBagConstraints.REMAINDER;
		c.weightx = 1.0;
		layout.setConstraints(this.passwordField, c);
		panel.add(this.passwordField);

		// add buttons
		panel = new JPanel();
		panel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
		mainPanel.add(panel, BorderLayout.SOUTH);

		panel.add(Box.createHorizontalGlue());

		button = new JButton("OK");
		button.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				SelectUserDialog.this.dialog.setVisible(false);
				SelectUserDialog.this.dialog.dispose();
				SelectUserDialog.this.user = SelectUserDialog.this.userField
						.getText();
				SelectUserDialog.this.pwd = new String(
						SelectUserDialog.this.passwordField.getPassword());
			}
		});
		panel.add(button);

		panel.add(Box.createHorizontalGlue());

		button = new JButton("Cancel");
		button.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				SelectUserDialog.this.dialog.setVisible(false);
				SelectUserDialog.this.dialog.dispose();
			}
		});
		panel.add(button);

		panel.add(Box.createHorizontalGlue());

		this.dialog.pack();
	}

	/**
	 * Get selected user
	 * 
	 * @return User-name
	 */
	public String getUser() {
		return this.user;
	}

	/**
	 * Get password to selected user
	 * 
	 * @return Password
	 */
	public String getPassword() {
		return this.pwd;
	}
}
