/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.gui.view.component;

import java.io.File;
import java.rmi.RemoteException;
import java.util.Arrays;

import javax.swing.JTable;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.AbstractTableModel;

import at.lcm.bu21.magopt.structure.IComponent;

/**
 * Table model for linked file of component
 * 
 * @author Werner Koppelsttter
 */
public class LinkedFilesModel extends AbstractTableModel {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/**
	 * Component to display
	 */
	private IComponent _component = null;

	/**
	 * Set component to display linked files
	 * 
	 * @param component
	 *            ParamComponent
	 */
	public void setComponent(IComponent component) {
		this._component = component;

		// refresh table of model
		for (TableModelListener listener : this
				.getListeners(TableModelListener.class)) {
			if ((listener instanceof JTable)
					&& this.equals(((JTable) listener).getModel())) {
				listener.tableChanged(new TableModelEvent(this));
			}
		}
	}

	@Override
	public int getColumnCount() {
		return LinkedFilesColumn.values().length;
	}

	@Override
	public int getRowCount() {
		int len = 0;

		if (this._component != null) {
			try {
				len = this._component.listLinkedFiles().length;
			} catch (RemoteException e) {
			}
		}

		return len;
	}

	@Override
	public Object getValueAt(int row, int column) {
		Object obj = null;
		File[] list;
		boolean exist;

		if (this._component != null) {
			try {
				// sort list
				list = this._component.listLinkedFiles();
				Arrays.sort(list);

				if (column == LinkedFilesColumn.FILE.getValue()) {
					obj = this._component.getRelLinkedFile(list[row]);
				} else if (column == LinkedFilesColumn.EXIST.getValue()) {
					exist = list[row].isAbsolute();
					exist &= list[row].exists();
					obj = exist;
				} else {
					obj = list[row];
				}
			} catch (RemoteException e) {
			}
		}

		return obj;
	}

	@Override
	public String getColumnName(int column) {
		for (LinkedFilesColumn e : LinkedFilesColumn.values()) {
			if (e.getValue() == column) {
				return e.getName();
			}
		}
		return null;
	}

	@Override
	public boolean isCellEditable(int row, int column) {
		return false;
	}

	@Override
	public Class<?> getColumnClass(int column) {
		Class<?> type = String.class;

		if (column == LinkedFilesColumn.EXIST.getValue()) {
			type = Boolean.class;
		}

		return type;
	}
}
