/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.gui.view.details;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.rmi.RemoteException;

import javax.swing.GroupLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import at.lcm.bu21.magopt.structure.IParamFile;
import at.lcm.bu21.magopt.structure.Location;
import magopt.gui.view.ParamBaseNoScroll;

/**
 * Displays ParamFile specific Data:<br>
 * - Directory<br>
 * - File Name<br>
 * - Imports
 * 
 * @author Aigner Markus
 * 
 */
public class ParamFileView extends ParamBaseNoScroll
		implements ActionListener, FocusListener {
	private static final long serialVersionUID = 1L;

	private JTextField mName;
	private JTextField mDescription;
	private JTextField mComment;

	private JComboBox<Location> mLocation;
	private JTextField mDirName;
	private JTextField mFileName;
	private JButton mSelector;
	protected JCheckBox mConstant;

	protected JButton bShow;

	protected IParamFile mPFile;

	private boolean update = false;

	public ParamFileView() {
		JPanel mainPane = new JPanel();
		this.setLayout(new BorderLayout());
		this.add(mainPane);
		mainPane.setPreferredSize(this.getSize()); /* TODO ? */
		GroupLayout layout = new GroupLayout(mainPane);
		mainPane.setLayout(layout);
		layout.setAutoCreateGaps(true);
		layout.setAutoCreateContainerGaps(true);

		/* name */
		JLabel lName = new JLabel("Name:");
		this.mName = new JTextField();
		this.mName.addActionListener(this);
		this.mName.addFocusListener(this);

		/* description */
		JLabel lDesc = new JLabel("Description:");
		this.mDescription = new JTextField();
		this.mDescription.addActionListener(this);
		this.mDescription.addFocusListener(this);

		/* comment */
		JLabel lComment = new JLabel("Comment:");
		this.mComment = new JTextField();
		this.mComment.addActionListener(this);
		this.mComment.addFocusListener(this);

		/* location */
		JLabel lLoc = new JLabel("Location:");
		this.mLocation = new JComboBox<Location>(Location.fileValues());
		this.mLocation.addActionListener(this);

		/* direcotry */
		JLabel lDirName = new JLabel("Directory:");
		this.mDirName = new JTextField();
		this.mDirName.addActionListener(this);
		this.mDirName.addFocusListener(this);

		/* file */
		JLabel lFileName = new JLabel("File:");
		this.mFileName = new JTextField();
		this.mFileName.addActionListener(this);
		this.mFileName.addFocusListener(this);

		/* selector */
		this.mSelector = new JButton("...");
		this.mSelector.addActionListener(this);

		/* show */
		this.bShow = new JButton("Show");
		this.bShow.addActionListener(this);
		// this.bShow.addFocusListener(this);

		/* constant flag */
		JLabel lConstant = new JLabel();
		this.mConstant = new JCheckBox("Constant");
		this.mConstant.addActionListener(this);

		/* config group layout */
		layout.setHorizontalGroup(layout.createSequentialGroup()
				.addGroup(layout
						.createParallelGroup(GroupLayout.Alignment.LEADING)
						.addComponent(lName).addComponent(lDesc)
						.addComponent(lComment).addComponent(lLoc)
						.addComponent(lDirName).addComponent(lFileName)
						.addComponent(lConstant).addComponent(lConstant))
				.addGroup(layout
						.createParallelGroup(GroupLayout.Alignment.LEADING)
						.addComponent(this.mName)
						.addComponent(this.mDescription)
						.addComponent(this.mComment)
						.addComponent(this.mLocation)
						.addComponent(this.mDirName)
						.addGroup(layout.createSequentialGroup()
								.addComponent(this.mFileName)
								.addComponent(this.mSelector))
						.addComponent(this.bShow)
						.addComponent(this.mConstant)));
		layout.setVerticalGroup(layout.createSequentialGroup()
				.addGroup(layout
						.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(lName).addComponent(this.mName))
				.addGroup(layout
						.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(lDesc).addComponent(this.mDescription))
				.addGroup(layout
						.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(lComment).addComponent(this.mComment))
				.addGroup(layout
						.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(lLoc).addComponent(this.mLocation))
				.addGroup(layout
						.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(lDirName).addComponent(this.mDirName))
				.addGroup(layout
						.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(lFileName).addComponent(this.mFileName)
						.addComponent(this.mSelector))
				.addGroup(layout
						.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(this.bShow))
				.addGroup(layout
						.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(lConstant).addComponent(this.mConstant)));
	}

	public void setFile(IParamFile pFile) {
		this.mPFile = pFile;
		this.update();
	}

	@Override
	public void focusGained(FocusEvent arg0) {
		/* do nothing */
	}

	@Override
	public void focusLost(FocusEvent e) {
		this.doWork(e.getSource());
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		this.doWork(e.getSource());
	}

	private void doWork(Object obj) {
		if (!this.update) {
			try {
				if (obj == this.mName) {
					this.mPFile.setName(this.mName.getText());
				} else if (obj == this.mDescription) {
					this.mPFile.setDescription(this.mDescription.getText());
				} else if (obj == this.mComment) {
					this.mPFile.setComment(this.mComment.getText());
				} else if (obj == this.mLocation) {
					ParamFileView.this.mPFile.changeLocation(
							(Location) this.mLocation.getSelectedItem());
				} else if (obj == this.mDirName) {
					ParamFileView.this.mPFile
							.setRelDirPath(this.mDirName.getText());
				} else if (obj == this.mFileName) {
					ParamFileView.this.mPFile
							.setFileName(this.mFileName.getText());
				} else if (obj == this.mSelector) {
					JFileChooser chooser = new JFileChooser();
					chooser.setDialogType(JFileChooser.OPEN_DIALOG);
					chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
					chooser.setFileHidingEnabled(true);
					chooser.setCurrentDirectory(
							this.mPFile.getAbsoluteDirectory());
					int returnVal = chooser.showOpenDialog(this);
					if (returnVal == JFileChooser.APPROVE_OPTION) {
						ParamFileView.this.mPFile
								.setAbsoluteFile(chooser.getSelectedFile());
					}
				} else if (obj == this.bShow) {
					this.mPFile.show();
				} else if (obj == this.mConstant) {
					this.mPFile.setConstant(this.mConstant.isSelected());
				}
			} catch (RemoteException ex) {
				ex.printStackTrace();
			} catch (Exception ex) {
				try {
					this.mPFile.error(ex.getMessage(), ex.getStackTrace());
				} catch (RemoteException e) {
					e.printStackTrace();
				}
			}

			this.update();
			this.fireParamChanged();
		}
	}

	@Override
	protected void update() {
		try {
			this.mName.setText(this.mPFile.getName());
			this.mDescription.setText(this.mPFile.getDescription());
			this.mComment.setText(this.mPFile.getComment());
			// FIXME Update
			this.update = true;
			this.mLocation.setSelectedItem(this.mPFile.getLocation());
			this.update = false;
			this.mDirName.setText(this.mPFile.getRelDirPath());
			this.mFileName.setText(this.mPFile.getFileName());

			this.mDirName.setEditable(!this.mPFile.getReadOnly());
			this.mFileName.setEditable(!this.mPFile.getReadOnly());
			this.mSelector.setEnabled(!this.mPFile.getReadOnly());
			this.mConstant.setSelected(this.mPFile.getConstant());
		} catch (RemoteException e) {
			e.printStackTrace();
		}
	}
}
