/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.gui.view.details;

import java.awt.BorderLayout;
import java.awt.Desktop;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.io.IOException;
import java.rmi.RemoteException;

import javax.swing.GroupLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import magopt.gui.view.ParamBaseNoScroll;
import at.lcm.bu21.magopt.structure.IProject;

/**
 * Displays Project specific Data:<br>
 * - Length<br>
 * - Current Index<br>
 * - Dataset Comment<br>
 * - Event Log Length<br>
 * - Project File<br>
 * - Project Directory
 * 
 * @author Aigner Markus
 * 
 */
public class ProjectView extends ParamBaseNoScroll implements ActionListener,
		FocusListener {
	private static final long serialVersionUID = 1L;

	private JTextField mName;
	private JTextField mDescription;
	private JTextField mComment;

	private JTextField mLen;
	private JTextField mIndex;
	private JTextField mDsComment;
	private JTextField mEventLen;
	private JTextField mFileName;
	private JTextField mDirName;

	private JButton bOpenDSDir;
	private IProject mProject;

	public ProjectView() {
		JPanel mainPane = new JPanel();
		this.setLayout(new BorderLayout());
		this.add(mainPane);

		GroupLayout layout = new GroupLayout(mainPane);
		mainPane.setLayout(layout);
		layout.setAutoCreateGaps(true);
		layout.setAutoCreateContainerGaps(true);

		/* name */
		JLabel lName = new JLabel("Name:");
		this.mName = new JTextField();
		this.mName.addActionListener(this);
		this.mName.addFocusListener(this);

		/* description */
		JLabel lDesc = new JLabel("Description:");
		this.mDescription = new JTextField();
		this.mDescription.addActionListener(this);
		this.mDescription.addFocusListener(this);

		/* comment */
		JLabel lComment = new JLabel("Comment:");
		this.mComment = new JTextField();
		this.mComment.addActionListener(this);
		this.mComment.addFocusListener(this);

		/* length */
		JLabel lLen = new JLabel("Length:");
		this.mLen = new JTextField();
		this.mLen.setEditable(false);

		/* index */
		JLabel lIndex = new JLabel("Current Index:");
		this.mIndex = new JTextField();
		this.mIndex.setEditable(false);

		/* ds-comment */
		JLabel lDsComment = new JLabel("Dataset Comment:");
		this.mDsComment = new JTextField();
		this.mDsComment.addActionListener(this);
		this.mDsComment.addFocusListener(this);

		/* event length */
		JLabel lEventLen = new JLabel("Event Length:");
		this.mEventLen = new JTextField();
		this.mEventLen.addActionListener(this);
		this.mEventLen.addFocusListener(this);

		/* project file */
		JLabel lFileName = new JLabel("Project File:");
		this.mFileName = new JTextField();
		this.mFileName.setEditable(false);

		/* project directory */
		JLabel lDirName = new JLabel("Project Directory:");
		this.mDirName = new JTextField();
		this.mDirName.setEditable(false);

		/* project directory */
		this.bOpenDSDir = new JButton("Open DS-Directory");
		this.bOpenDSDir.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				try {
					Desktop.getDesktop().open(
							ProjectView.this.mProject.getFilesDirectory());
				} catch (RemoteException e1) {
					e1.printStackTrace();
				} catch (IOException e1) {
					e1.printStackTrace();
				}
			}
		});

		/* setup group layout */
		layout.setHorizontalGroup(layout
				.createSequentialGroup()
				.addGroup(
						layout.createParallelGroup(
								GroupLayout.Alignment.LEADING)
								.addComponent(lName).addComponent(lDesc)
								.addComponent(lComment).addComponent(lLen)
								.addComponent(lIndex).addComponent(lDsComment)
								.addComponent(lEventLen)
								.addComponent(lFileName).addComponent(lDirName))
				.addGroup(
						layout.createParallelGroup(
								GroupLayout.Alignment.LEADING)
								.addComponent(this.mName)
								.addComponent(this.mDescription)
								.addComponent(this.mComment)
								.addComponent(this.mLen)
								.addComponent(this.mIndex)
								.addComponent(this.mDsComment)
								.addComponent(this.mEventLen)
								.addComponent(this.mFileName)
								.addComponent(this.mDirName)
								.addComponent(this.bOpenDSDir)));
		layout.setVerticalGroup(layout
				.createSequentialGroup()
				.addGroup(
						layout.createParallelGroup(
								GroupLayout.Alignment.BASELINE)
								.addComponent(lName).addComponent(this.mName))
				.addGroup(
						layout.createParallelGroup(
								GroupLayout.Alignment.BASELINE)
								.addComponent(lDesc)
								.addComponent(this.mDescription))
				.addGroup(
						layout.createParallelGroup(
								GroupLayout.Alignment.BASELINE)
								.addComponent(lComment)
								.addComponent(this.mComment))
				.addGroup(
						layout.createParallelGroup(
								GroupLayout.Alignment.BASELINE)
								.addComponent(lLen).addComponent(this.mLen))
				.addGroup(
						layout.createParallelGroup(
								GroupLayout.Alignment.BASELINE)
								.addComponent(lIndex).addComponent(this.mIndex))
				.addGroup(
						layout.createParallelGroup(
								GroupLayout.Alignment.BASELINE)
								.addComponent(lDsComment)
								.addComponent(this.mDsComment))
				.addGroup(
						layout.createParallelGroup(
								GroupLayout.Alignment.BASELINE)
								.addComponent(lEventLen)
								.addComponent(this.mEventLen))
				.addGroup(
						layout.createParallelGroup(
								GroupLayout.Alignment.BASELINE)
								.addComponent(lFileName)
								.addComponent(this.mFileName))
				.addGroup(
						layout.createParallelGroup(
								GroupLayout.Alignment.BASELINE)
								.addComponent(lDirName)
								.addComponent(this.mDirName))
				.addGroup(
						layout.createParallelGroup(
								GroupLayout.Alignment.BASELINE).addComponent(
								this.bOpenDSDir)));
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		this.doWork(e.getSource());
	}

	@Override
	public void focusGained(FocusEvent e) {
		/* do nothing */
	}

	@Override
	public void focusLost(FocusEvent e) {
		this.doWork(e.getSource());
	}

	private void doWork(Object obj) {
		try {
			if (obj == this.mName) {
				this.mProject.setName(this.mName.getText());
			} else if (obj == this.mDescription) {
				this.mProject.setDescription(this.mDescription.getText());
			} else if (obj == this.mComment) {
				this.mProject.setComment(this.mComment.getText());
			} else if (obj == this.mDsComment) {
				this.mProject.setDSComment(this.mDsComment.getText());
			} else if (obj == this.mEventLen) {
				this.mProject.getEventLog().setMaxSize(
						Integer.valueOf(this.mEventLen.getText()));
			}
		} catch (RemoteException ex) {
			ex.printStackTrace();
		} catch (Exception ex) {
			try {
				this.mProject.error(ex.getMessage(), ex.getStackTrace());
			} catch (RemoteException e) {
				e.printStackTrace();
			}
		}

		this.update();
		this.fireParamChanged();
	}

	public void setProject(IProject newProject) {
		this.mProject = newProject;
		this.update();
	}

	@Override
	protected void update() {
		try {
			this.mName.setText(this.mProject.getName());
			this.mDescription.setText(this.mProject.getDescription());
			this.mComment.setText(this.mProject.getComment());
			this.mLen.setText(String.valueOf(this.mProject.getLength()));
			this.mIndex.setText(String.valueOf(this.mProject.getIndex() + 1));
			this.mDsComment.setText(this.mProject.getDSComment());
			this.mEventLen.setText(String.valueOf(this.mProject.getEventLog()
					.getMaxSize()));
			this.mFileName.setText(this.mProject.getMopFile().getName());
			this.mDirName.setText(this.mProject.getMopFile().getParent());
		} catch (RemoteException e) {
			e.printStackTrace();
		}
	}

}
