/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.gui.view.port;

import java.awt.Component;
import java.rmi.RemoteException;

import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;

import at.lcm.bu21.magopt.structure.IDependItem;
import at.lcm.bu21.magopt.structure.IParameter;
import at.lcm.bu21.magopt.structure.ParamField;

/**
 * Cell-renderer for port-table
 * 
 * @author Werner Koppelsttter
 */
public class PortRenderer extends DefaultTableCellRenderer {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	@Override
	public Component getTableCellRendererComponent(JTable table, Object value,
			boolean isSelected, boolean hasFocus, int row, int column) {
		String tt;
		IParameter param;
		IDependItem item;

		super.getTableCellRendererComponent(table, value, isSelected, hasFocus,
				row, column);

		item = (IDependItem) table.getModel().getValueAt(row, -1);
		try {
			if (column == PortColumn.VARIABLE.getValue()) {
				tt = "<html>";
				tt += this.getText();
				tt += "<br><i>Description:</i> "
						+ item.getVarDesc().getDescription();
				tt += "<br><i>Unit:</i> " + item.getVarDesc().getUnit();
				tt += "<br><i>Optional:</i> " + item.getVarDesc().getOptional();
				tt += "<!html>";
			} else if (column == PortColumn.PARAMETER.getValue()) {
				param = item.getHandle();
				if (param != null) {
					tt = "<html>";
					tt += this.getText();
					tt += "<br><i>Identifier:</i> " + param.getIdentifier();
					tt += "<br><i>Data:</i> " + param.getData();
					if (param instanceof ParamField) {
						tt += "<br><i>Unit:</i> "
								+ ((ParamField) param).getUnit();
					}
					tt += "<!html>";
				} else {
					tt = this.getText();
				}
			} else {
				tt = null;
			}
			if (tt.length() == 0) {
				tt = null;
			}
		} catch (RemoteException e) {
			tt = null;
			e.printStackTrace();
		}

		this.setToolTipText(tt);

		return this;
	}
}
