/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 247 $
 * $Date: 2020-03-19 10:13:52 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.optim.core.mapping.utils.dstructures;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class TMappingTaskBatch {
	private final List<TMappingTask> taskList = new ArrayList<TMappingTask>();

	public TMappingTaskBatch() {
	}

	public TMappingTaskBatch(String fileName) throws Exception {
		this.appendFromFile(fileName);
	}

	public void appendFromFile(String fileName) throws Exception {
		BufferedReader br = new BufferedReader(new FileReader(fileName));
		String line;
		while (((line = br.readLine()) != null) && (line.length() > 0)) {
			TMappingTask newTask = new TMappingTask();
			String[] lineTokens = line.split(TMappingConstants.PARAM_ARRAY_SPLIT);
			for (int i = 0; i < lineTokens.length; i++) {
				String[] keyValuePair = lineTokens[i].split(TMappingConstants.KEY_VALUE_SPLIT);
				if (TMappingTask.TParameter.contains(keyValuePair[0])) {
					newTask.getParameters().put(keyValuePair[0], keyValuePair[1]);
				} else if (TMappingTask.TRunInfo.contains(keyValuePair[0])) {
					newTask.getRunInfo().put(keyValuePair[0], keyValuePair[1]);
				} else if (TMappingTask.TResult.contains(keyValuePair[0])) {
					newTask.getResults().put(keyValuePair[0], keyValuePair[1]);
				} else {
					throw new Exception(
							"ERROR: undefined test setting: " + "|" + keyValuePair[0] + "|=|" + keyValuePair[1] + "|");
				}
			}
			this.taskList.add(newTask);
		}
		br.close();
	}

	public List<TMappingTask> getTaskList() {
		return this.taskList;
	}

	public void saveRecordToFile(String fileName, boolean rewriteFile, int recordIndex) throws Exception {
		File f = new File(fileName);
		if (f.getParent() != null) {
			File dir = new File(f.getParent());
			if (!dir.exists() && !dir.mkdirs()) {
				throw new IOException("ERROR: could not create directory path: " + f.getParent());
			}
		}
		if (f.exists() && rewriteFile) {
			f.delete();
		}

		BufferedWriter bw = new BufferedWriter(new FileWriter(fileName, !rewriteFile));
		bw.write(this.taskList.get(recordIndex).toString() + "\n");
		bw.close();
	}

	public void saveSetToFile(String fileName, boolean rewriteFile) throws Exception {
		File f = new File(fileName);
		if (f.getParent() != null) {
			File dir = new File(f.getParent());
			if (!dir.exists() && !dir.mkdirs()) {
				throw new IOException("ERROR: could not create directory path: " + f.getParent());
			}
		}
		if (f.exists()) {
			f.delete();
		}

		BufferedWriter bw = new BufferedWriter(new FileWriter(fileName, !rewriteFile));
		String line;
		for (TMappingTask task : this.taskList) {
			line = task.toString() + "\n";
			bw.write(line);
		}
		bw.close();
	}

}
