/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 247 $
 * $Date: 2020-03-19 10:13:52 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.optim.gui.mapping;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.SystemColor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.util.HashMap;
import java.util.Map;

import javax.swing.DefaultComboBoxModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JTextField;
import javax.swing.border.LineBorder;

import at.lcm.bu21.optim.core.Individual;
import at.lcm.bu21.optim.core.Optimization;
import at.lcm.bu21.optim.core.ResultDesc;
import at.lcm.bu21.optim.core.VariableDesc;
import at.lcm.bu21.optim.core.mapping.MappingManager;
import at.lcm.bu21.optim.core.mapping.utils.dstructures.TMappingConstants;
import at.lcm.bu21.optim.core.mapping.utils.dstructures.TMappingConstants.TThreadState;

public class PanelSensitivity extends JPanel {

	class SensitivityComboBoxActionLister implements ActionListener {

		@Override
		public void actionPerformed(ActionEvent event) {

			// System.out.println(PanelSensitivity.this.selectAnalysisComboBox
			// .getSelectedIndex()
			// + " "
			// + PanelSensitivity.this.parentMappingManager
			// .getCurrentSensitivityAnalysisDisplayIndex());

			if (PanelSensitivity.this.selectAnalysisComboBox
					.getSelectedIndex() != PanelSensitivity.this.parentMappingManager
							.getCurrentSensitivityAnalysisDisplayIndex()) {
				PanelSensitivity.this.parentMappingManager
						.selectSavedSensitvityAnalysis(PanelSensitivity.this.selectAnalysisComboBox.getSelectedIndex());
			}
		}

	}

	class SensitivityControlButtonActionLister implements ActionListener {

		private final String actionToPerform;

		private SensitivityControlButtonActionLister(String actionToPerform) {
			this.actionToPerform = actionToPerform;
		}

		@Override
		public void actionPerformed(ActionEvent event) {

			if (this.actionToPerform.equals("RUN-MAP")) {

				for (VariableDesc vDesc : PanelSensitivity.this.parentMappingManager.getProblem()
						.getVariedVariables()) {
					PanelSensitivity.this.parentMappingManager.setPlusVariableRangesPreference(vDesc.getName(),
							PanelSensitivity.this.plusVariableRangesTextFields.get(vDesc.getName()).getText());
					PanelSensitivity.this.parentMappingManager.setMinusVariableRangesPreference(vDesc.getName(),
							PanelSensitivity.this.minusVariableRangesTextFields.get(vDesc.getName()).getText());
				}

				PanelSensitivity.this.parentMappingManager.runMappingBasedSensitivityAnalysis();
			}
			if (this.actionToPerform.equals("RUN-FE")) {
				PanelSensitivity.this.parentMappingManager.runFEBasedSensitivityAnalysis();
			}
			if (this.actionToPerform.equals("STOP")) {
				PanelSensitivity.this.parentMappingManager.stopSensitivityAnalysis();
			}
			if (this.actionToPerform.equals("SAVE")) {
				PanelSensitivity.this.parentMappingManager.saveMappingBasedSensitivityInformation();
			}
			if (this.actionToPerform.equals("DISCARD")) {
				PanelSensitivity.this.parentMappingManager.discardMappingBasedSensitivityInformation();
			}
			if (this.actionToPerform.equals("CHECK-FE")) {
				PanelSensitivity.this.parentMappingManager.checkFEProgress();
			}
			if (this.actionToPerform.equals("RUN-AWSA")) {
				PanelSensitivity.this.parentMappingManager
						.runAWSA(PanelSensitivity.this.archiveSensitivityPrefixTextField.getText());
			}
			if (this.actionToPerform.equals("USE-MAP-COLOR")) {
				PanelSensitivity.this.parentMappingManager.switchToMappingColors();
			}
			if (this.actionToPerform.equals("USE-FE-COLOR")) {
				PanelSensitivity.this.parentMappingManager.switchToFEColors();
			}

		}
	}

	private JTextField textField;
	private JCheckBox chckbxNewCheckBox;

	private MappingManager parentMappingManager;

	/**
	 * Create panel. Used by WindowsBuilder
	 */

	private JPanel selectExistingAnalysisPanel;
	private JPanel designPanel;
	private JPanel designVariableVariationRangesPanel;
	private JPanel targetVariationRangesPanel;
	private JPanel sensitivityRunPanel;
	private JPanel sensitivityResultsPanel;
	private JPanel archiveSensitivityRunPanel;

	private JLabel selectExistingAnalysisTitleLabel = new JLabel();
	private JComboBox selectAnalysisComboBox = new JComboBox();
	private JLabel selectedDesignTitleLabel = new JLabel();
	private JLabel originalPerformanceTitleLabel = new JLabel();
	private JLabel variableRangesTitleLabel = new JLabel();
	private JLabel plusVariableRangesTitleLabel = new JLabel();
	private JLabel minusVariableRangesTitleLabel = new JLabel();
	private JLabel targetRangesTitleLabel = new JLabel();
	private JLabel plusTargetTitleLabel = new JLabel();
	private JLabel minusTargetTitleLabel = new JLabel();
	private JLabel localSensitivityResultsTitleLabel = new JLabel();
	private JLabel globalSensitivityResultsTitleLabel = new JLabel();
	private Map<String, JLabel> designVariableLabels = new HashMap<String, JLabel>();
	private Map<String, JLabel> targetLabels = new HashMap<String, JLabel>();
	private Map<String, JTextField> designVariableFields = new HashMap<String, JTextField>();
	private Map<String, JTextField> targetFields = new HashMap<String, JTextField>();
	private Map<String, JTextField> plusVariableRangesTextFields = new HashMap<String, JTextField>();
	private Map<String, JTextField> minusVariableRangesTextFields = new HashMap<String, JTextField>();
	private Map<String, JTextField> plusTargetRangesTextFields = new HashMap<String, JTextField>();
	private Map<String, JTextField> minusTargetRangesTextFields = new HashMap<String, JTextField>();
	private Map<String, JButton> sensitivityRunControlButtons = new HashMap<String, JButton>();
	private JProgressBar sensitivityRunProgressBar;
	private Map<String, Map<String, JTextField>> localSensitivityTextFields = new HashMap<String, Map<String, JTextField>>();
	private Map<String, JTextField> globalSensitivityTextFields = new HashMap<String, JTextField>();
	private Map<String, JLabel> localSensitivityDesignVariableLabels = new HashMap<String, JLabel>();
	private Map<String, JLabel> localSensitivityTargetLabels = new HashMap<String, JLabel>();
	private BufferedImage colorScaleImage;
	private JLabel imageLabel;
	private JLabel maxColorScaleTitleLabel = new JLabel();
	private JLabel minColorScaleTitleLabel = new JLabel();
	private JButton archiveSensitivityRunButton = new JButton();
	private JTextField archiveSensitivityPrefixTextField = new JTextField();
	private JProgressBar archiveSensitivityRunProgressBar = new JProgressBar();
	private JButton useMappingColoring = new JButton();
	private JButton useFEColoring = new JButton();

	private double sensitivityDefaultRange = 0.02;

	private PanelSensitivity() throws Exception {
		this.textField = new JTextField();
		this.textField.setBackground(new Color(240, 240, 240));
		this.textField.setText("5000000001");
		this.add(this.textField);
		this.textField.setColumns(10);

		JCheckBox chckbxNewCheckBox = new JCheckBox("FT");
		chckbxNewCheckBox.setSelected(true);
		this.add(chckbxNewCheckBox);
	}

	public PanelSensitivity(Optimization optim) {
		this.parentMappingManager = optim.getMappingControl();
		this.setLayout(null);

		this.generateUIComponents();
		this.refresh();
	}

	public void generateUIComponents() {
		int outerWidth = 20;
		int innerWidth = 10;
		int outerHeight = 20;
		int innerHeight = 10;
		int panelStartX = 20;
		int selectExistingAnalysisPanelStartY = 20;
		int designPanelStartY = 20;
		int designPanelWidth = 0;
		int designPanelHeight = 0;
		int designVariableVariationRangesPanelStartY = 0;
		int designVariableVariationRangesPanelWidth = 0;
		int designVariableVariationRangesPanelHeight = 0;
		int targetVariationRangesPanelStartX = 0;
		int targetVariationRangesPanelWidth = 0;
		int sensitivityRunPanelStartY = 0;
		int sensitivityRunPanelWidth = 0;
		int sensitivityRunPanelHeight = 0;
		int sensitivityResultsPanelStartY = 0;
		int sensitivityResultsPanelWidth = 0;
		int sensitivityResultsPanelHeight = 0;
		int archiveSensitivityRunPanelStartY = 0;
		int archiveSensitivityRunPanelWidth = 0;
		int archiveSensitivityRunPanelHeight = 0;

		int labelWidth = 100;
		int labelHeight = 20;
		int longLabelWidth = 350;

		int titleLabelStartX = outerWidth;
		int titleLabelStartY = innerHeight;
		int titleLabelWidth = 150;
		int titleLabelHeight = 20;
		int plusMinusLabelWidth = 10;
		int maxMinLabelWidth = 10;
		int originalPerformanceLabelStartX = titleLabelWidth + (2 * outerWidth);

		int buttonStartX = titleLabelStartX + innerWidth + plusMinusLabelWidth;
		int buttonStartY = 20;
		int buttonWidth = (2 * labelWidth) + innerWidth;
		int buttonHeight = 30;
		int sensitivityMatrixWidth = (this.parentMappingManager.getSensitivityTargetNames().size() + 1)
				* (labelWidth + innerWidth);
		int sensitivityMatrixHeight = (this.parentMappingManager.getProblem().getVariedVariables().length + 1)
				* (labelHeight + innerHeight);

		// panel title labels
		this.selectedDesignTitleLabel = new JLabel("Selected design:");
		this.selectedDesignTitleLabel.setBounds(titleLabelStartX, titleLabelStartY, titleLabelWidth, titleLabelHeight);
		this.selectedDesignTitleLabel.setFont(new Font("Tahoma", Font.BOLD, 11));

		this.variableRangesTitleLabel = new JLabel("Sensitivity ranges (nominal values):");
		this.variableRangesTitleLabel.setBounds(titleLabelStartX, titleLabelStartY, 250, titleLabelHeight);
		this.variableRangesTitleLabel.setFont(new Font("Tahoma", Font.BOLD, 11));

		this.plusVariableRangesTitleLabel = new JLabel("+");
		this.plusVariableRangesTitleLabel.setBounds(outerWidth, titleLabelStartY + titleLabelHeight + innerHeight,
				plusMinusLabelWidth, titleLabelHeight);
		this.plusVariableRangesTitleLabel.setFont(new Font("Tahoma", Font.BOLD, 11));

		this.minusVariableRangesTitleLabel = new JLabel("-");
		this.minusVariableRangesTitleLabel.setBounds(outerWidth + 2,
				titleLabelStartY + titleLabelHeight + innerHeight + labelHeight + innerHeight, plusMinusLabelWidth,
				titleLabelHeight);
		this.minusVariableRangesTitleLabel.setFont(new Font("Tahoma", Font.BOLD, 11));

		this.targetRangesTitleLabel = new JLabel("Estimated performance ranges (nominal values):");
		this.targetRangesTitleLabel.setBounds(titleLabelStartX, titleLabelStartY, longLabelWidth, titleLabelHeight);
		this.targetRangesTitleLabel.setFont(new Font("Tahoma", Font.BOLD, 11));

		this.plusTargetTitleLabel = new JLabel("+");
		this.plusTargetTitleLabel.setBounds(outerWidth, titleLabelStartY + titleLabelHeight + innerHeight,
				maxMinLabelWidth, titleLabelHeight);
		this.plusTargetTitleLabel.setFont(new Font("Tahoma", Font.BOLD, 11));

		this.minusTargetTitleLabel = new JLabel("-");
		this.minusTargetTitleLabel.setBounds(outerWidth + 2,
				titleLabelStartY + titleLabelHeight + innerHeight + labelHeight + innerHeight, maxMinLabelWidth,
				titleLabelHeight);
		this.minusTargetTitleLabel.setFont(new Font("Tahoma", Font.BOLD, 11));

		this.localSensitivityResultsTitleLabel = new JLabel("Local sensitivity estimates (relative values):");
		this.localSensitivityResultsTitleLabel.setBounds(titleLabelStartX, titleLabelStartY, longLabelWidth,
				titleLabelHeight);
		this.localSensitivityResultsTitleLabel.setFont(new Font("Tahoma", Font.BOLD, 11));

		this.globalSensitivityResultsTitleLabel = new JLabel("Global sensitivity estimates (relative values):");
		this.globalSensitivityResultsTitleLabel.setBounds(titleLabelStartX,
				titleLabelStartY + outerHeight + innerHeight + sensitivityMatrixHeight + labelHeight, longLabelWidth,
				titleLabelHeight);
		this.globalSensitivityResultsTitleLabel.setFont(new Font("Tahoma", Font.BOLD, 11));

		this.selectExistingAnalysisTitleLabel = new JLabel("Select saved sensitivity analysis:");
		this.selectExistingAnalysisTitleLabel.setBounds(buttonStartX, buttonStartY, buttonWidth, buttonHeight);
		this.selectExistingAnalysisTitleLabel.setFont(new Font("Tahoma", Font.BOLD, 11));

		this.maxColorScaleTitleLabel = new JLabel("max");
		this.maxColorScaleTitleLabel.setFont(new Font("Tahoma", Font.BOLD, 11));

		this.minColorScaleTitleLabel = new JLabel("min");
		this.minColorScaleTitleLabel.setFont(new Font("Tahoma", Font.BOLD, 11));

		// the sensitivity control buttons
		// RUN-MAP
		JButton button = new JButton("Run new mapping-based analysis");
		button.setBounds(buttonStartX, buttonStartY, buttonWidth, buttonHeight);
		button.addActionListener(new SensitivityControlButtonActionLister("RUN-MAP"));
		this.sensitivityRunControlButtons.put("RUN-MAP", button);

		// STOP
		button = new JButton("Stop ");
		button.setBounds(buttonStartX, buttonStartY, buttonWidth, buttonHeight);
		button.addActionListener(new SensitivityControlButtonActionLister("STOP"));
		this.sensitivityRunControlButtons.put("STOP", button);

		// SAVE
		button = new JButton("Save mapping-based analysis");
		button.setBounds(buttonStartX, buttonStartY, buttonWidth, buttonHeight);
		button.addActionListener(new SensitivityControlButtonActionLister("SAVE"));
		this.sensitivityRunControlButtons.put("SAVE", button);

		// DISCARD
		button = new JButton("Discard mapping-based analysis");
		button.setBounds(buttonStartX + buttonWidth + innerWidth, buttonStartY, buttonWidth, buttonHeight);
		button.addActionListener(new SensitivityControlButtonActionLister("DISCARD"));
		this.sensitivityRunControlButtons.put("DISCARD", button);

		// RUN-FE
		button = new JButton("Run simulation-based analysis");
		button.setBounds(buttonStartX + buttonWidth + innerWidth, buttonStartY, buttonWidth, buttonHeight);
		button.addActionListener(new SensitivityControlButtonActionLister("RUN-FE"));
		this.sensitivityRunControlButtons.put("RUN-FE", button);

		// RUN-FE
		button = new JButton("Check sim. progress ...");
		button.setBounds(buttonStartX + buttonWidth + innerWidth, buttonStartY, buttonWidth, buttonHeight);
		button.addActionListener(new SensitivityControlButtonActionLister("CHECK-FE"));
		this.sensitivityRunControlButtons.put("CHECK-FE", button);

		// the design panel
		int count = 0;
		for (VariableDesc vDesc : this.parentMappingManager.getProblem().getVariedVariables()) {

			// the design variable labels
			String value = vDesc.getName().substring(vDesc.getName().lastIndexOf(".") + 1);
			JLabel label = new JLabel(value);
			label.setBounds(outerWidth + plusMinusLabelWidth + innerWidth + (count * (labelWidth + innerWidth)),
					titleLabelStartY + titleLabelHeight + innerHeight, labelWidth, labelHeight);
			label.setFont(new Font("Tahoma", Font.PLAIN, 11));
			this.designVariableLabels.put(vDesc.getName(), label);

			// the design variable fields
			JTextField textField = new JTextField("", 10);
			textField.setEditable(false);
			textField.setBounds(outerWidth + plusMinusLabelWidth + innerWidth + (count * (labelWidth + innerWidth)),
					titleLabelStartY + titleLabelHeight + innerHeight + labelHeight + innerHeight, labelWidth,
					labelHeight);
			textField.setFont(new Font("Tahoma", Font.PLAIN, 11));
			this.designVariableFields.put(vDesc.getName(), textField);

			count++;
			designPanelWidth = ((2 * outerWidth) + (count * (labelWidth + innerWidth))) + innerWidth
					+ plusMinusLabelWidth;
			designPanelHeight = titleLabelStartY + titleLabelHeight + innerHeight + labelHeight + labelHeight
					+ innerHeight + outerHeight;
		}

		// more panel title labels
		int adjustedStartX = Math.max(designPanelWidth + outerWidth, originalPerformanceLabelStartX) + outerWidth;
		this.originalPerformanceTitleLabel = new JLabel("Base performance:");
		this.originalPerformanceTitleLabel.setBounds(adjustedStartX - innerWidth, titleLabelStartY, titleLabelWidth,
				titleLabelHeight);
		this.originalPerformanceTitleLabel.setFont(new Font("Tahoma", Font.BOLD, 11));

		// more design panel
		count = 0;
		for (String targetName : this.parentMappingManager.getSensitivityTargetNames()) {

			// the target labels
			String value = targetName.substring(targetName.lastIndexOf(".") + 1);
			JLabel label = new JLabel(value);
			label.setBounds(adjustedStartX + maxMinLabelWidth + (count * (labelWidth + innerWidth)),
					titleLabelStartY + titleLabelHeight + innerHeight, labelWidth, labelHeight);
			label.setFont(new Font("Tahoma", Font.PLAIN, 11));
			this.targetLabels.put(targetName, label);

			// the target fields
			JTextField textField = new JTextField("", 10);
			textField.setEditable(false);
			textField.setBounds(adjustedStartX + maxMinLabelWidth + (count * (labelWidth + innerWidth)),
					titleLabelStartY + titleLabelHeight + innerHeight + labelHeight + innerHeight, labelWidth,
					labelHeight);
			textField.setFont(new Font("Tahoma", Font.PLAIN, 11));

			this.targetFields.put(targetName, textField);

			count++;
			designPanelWidth = adjustedStartX
					+ Math.max((count * (labelWidth + innerWidth)) + innerWidth + maxMinLabelWidth,
							outerWidth + innerWidth + longLabelWidth);
		}

		// the design variable variation ranges panel
		count = 0;
		for (VariableDesc vDesc : this.parentMappingManager.getProblem().getVariedVariables()) {

			// the plus range fields
			JTextField textField = new JTextField("", 10);
			textField.setEditable(true);
			textField.setBounds(outerWidth + plusMinusLabelWidth + innerWidth + (count * (labelWidth + innerWidth)),
					titleLabelStartY + titleLabelHeight + innerHeight, labelWidth, labelHeight);
			textField.setFont(new Font("Tahoma", Font.PLAIN, 11));
			this.plusVariableRangesTextFields.put(vDesc.getName(), textField);

			// the minus range fields
			textField = new JTextField("", 10);
			textField.setEditable(true);
			textField.setBounds(outerWidth + plusMinusLabelWidth + innerWidth + (count * (labelWidth + innerWidth)),
					titleLabelStartY + titleLabelHeight + innerHeight + labelHeight + innerHeight, labelWidth,
					labelHeight);
			textField.setFont(new Font("Tahoma", Font.PLAIN, 11));
			this.minusVariableRangesTextFields.put(vDesc.getName(), textField);

			count++;
			designVariableVariationRangesPanelWidth = ((2 * outerWidth) + (count * (labelWidth + innerWidth)))
					+ plusMinusLabelWidth;
			designVariableVariationRangesPanelHeight = titleLabelStartY + titleLabelHeight + innerHeight + labelHeight
					+ labelHeight + innerHeight + outerHeight;
		}

		// the target variation ranges panel
		count = 0;
		for (String targetName : this.parentMappingManager.getSensitivityTargetNames()) {

			// the max range fields
			JTextField textField = new JTextField("", 10);
			textField.setEditable(false);
			// textField.setBackground(Color.lightGray);
			textField.setBounds(outerWidth + maxMinLabelWidth + innerWidth + (count * (labelWidth + innerWidth)),
					titleLabelStartY + titleLabelHeight + innerHeight, labelWidth, labelHeight);
			textField.setFont(new Font("Tahoma", Font.PLAIN, 11));
			this.plusTargetRangesTextFields.put(targetName, textField);

			// the min range fields
			textField = new JTextField("", 10);
			textField.setEditable(false);
			// textField.setBackground(Color.lightGray);
			textField.setBounds(outerWidth + maxMinLabelWidth + innerWidth + (count * (labelWidth + innerWidth)),
					titleLabelStartY + titleLabelHeight + innerHeight + labelHeight + innerHeight, labelWidth,
					labelHeight);
			textField.setFont(new Font("Tahoma", Font.PLAIN, 11));
			this.minusTargetRangesTextFields.put(targetName, textField);

			count++;
		}

		// the sensitivity results panel
		int horizCount = 0;
		int vertCount = 0;
		for (VariableDesc vDesc : this.parentMappingManager.getProblem().getVariedVariables()) {

			Map<String, JTextField> targetTextFields = new HashMap<String, JTextField>();

			horizCount = 0;
			for (String targetName : this.parentMappingManager.getSensitivityTargetNames()) {
				// local sensitivity matrix
				JTextField textField = new JTextField("", 10);
				textField.setEditable(false);
				// textField.setBackground(Color.lightGray);
				int startX = ((horizCount + 1) * (labelWidth + innerWidth)) + outerWidth;
				int startY = ((vertCount + 1) * (labelHeight + innerHeight)) + (2 * innerHeight) + labelHeight;
				textField.setBounds(startX, startY, labelWidth, labelHeight);
				textField.setFont(new Font("Tahoma", Font.PLAIN, 11));
				targetTextFields.put(targetName, textField);

				if (vertCount == 0) {
					// local sensitivity target labels
					String value = targetName.substring(targetName.lastIndexOf(".") + 1);
					JLabel label = new JLabel(value);
					startX = outerWidth + labelWidth + innerWidth + (horizCount * (labelWidth + innerWidth));
					startY = (2 * innerHeight) + labelHeight;
					label.setBounds(startX, startY, labelWidth, labelHeight);
					label.setFont(new Font("Tahoma", Font.PLAIN, 11));
					this.localSensitivityTargetLabels.put(targetName, label);

					// global sensitivity fields
					textField = new JTextField("", 10);
					textField.setEditable(false);
					// textField.setBackground(Color.lightGray);
					startX = ((horizCount + 1) * (labelWidth + innerWidth)) + outerWidth;
					startY = this.globalSensitivityResultsTitleLabel.getY() + innerHeight + labelHeight;
					textField.setBounds(startX, startY, labelWidth, labelHeight);
					textField.setFont(new Font("Tahoma", Font.PLAIN, 11));
					this.globalSensitivityTextFields.put(targetName, textField);

					this.useMappingColoring = new JButton("M");
					startX = outerWidth;
					startY = this.globalSensitivityResultsTitleLabel.getY() + innerHeight + labelHeight;
					int smallButtonWidth = (labelWidth) / 2;
					int smallButtonHeight = labelHeight;
					this.useMappingColoring.setBounds(startX, startY, smallButtonWidth, smallButtonHeight);
					this.useMappingColoring
							.addActionListener(new SensitivityControlButtonActionLister("USE-MAP-COLOR"));

					this.useFEColoring = new JButton("S");
					startX = outerWidth + smallButtonWidth;
					startY = this.globalSensitivityResultsTitleLabel.getY() + innerHeight + labelHeight;
					this.useFEColoring.setBounds(startX, startY, smallButtonWidth, smallButtonHeight);
					this.useFEColoring.addActionListener(new SensitivityControlButtonActionLister("USE-FE-COLOR"));
				}

				// local sensitivity design variable labels
				String value = vDesc.getName().substring(vDesc.getName().lastIndexOf(".") + 1);
				JLabel label = new JLabel(value);
				startX = outerWidth;
				startY = (2 * innerHeight) + labelHeight + ((vertCount + 1) * (labelHeight + innerHeight));
				label.setBounds(startX, startY, labelWidth, labelHeight);
				label.setFont(new Font("Tahoma", Font.PLAIN, 11));
				this.localSensitivityDesignVariableLabels.put(vDesc.getName(), label);

				horizCount++;
			}
			this.localSensitivityTextFields.put(vDesc.getName(), targetTextFields);

			vertCount++;
		}

		// the panels
		designPanelWidth = Math.max((adjustedStartX + titleLabelWidth + innerWidth), designPanelWidth);

		this.designPanel = new JPanel();
		this.designPanel.setBorder(new LineBorder(new Color(0, 0, 0)));
		this.designPanel.setBounds(panelStartX, designPanelStartY, designPanelWidth, designPanelHeight);
		this.designPanel.setLayout(null);

		designVariableVariationRangesPanelStartY = designPanelStartY + designPanelHeight + outerHeight;
		this.designVariableVariationRangesPanel = new JPanel();
		this.designVariableVariationRangesPanel.setBorder(new LineBorder(new Color(0, 0, 0)));
		this.designVariableVariationRangesPanel.setBounds(panelStartX, designVariableVariationRangesPanelStartY,
				designVariableVariationRangesPanelWidth, designVariableVariationRangesPanelHeight);
		this.designVariableVariationRangesPanel.setLayout(null);

		targetVariationRangesPanelStartX = panelStartX + designVariableVariationRangesPanelWidth + outerWidth;
		targetVariationRangesPanelWidth = designPanelWidth - designVariableVariationRangesPanelWidth - outerWidth;
		this.targetVariationRangesPanel = new JPanel();
		this.targetVariationRangesPanel.setBorder(new LineBorder(new Color(0, 0, 0)));
		this.targetVariationRangesPanel.setBounds(targetVariationRangesPanelStartX,
				designVariableVariationRangesPanelStartY, targetVariationRangesPanelWidth,
				designVariableVariationRangesPanelHeight);
		this.targetVariationRangesPanel.setLayout(null);

		sensitivityRunPanelStartY = designVariableVariationRangesPanelStartY + designVariableVariationRangesPanelHeight
				+ outerHeight;
		sensitivityRunPanelWidth = Math.max((2 * (buttonWidth + outerHeight)) + innerWidth + plusMinusLabelWidth,
				designVariableVariationRangesPanelWidth);
		sensitivityRunPanelHeight = (2 * outerHeight) + buttonHeight;
		this.sensitivityRunPanel = new JPanel();
		this.sensitivityRunPanel.setBorder(new LineBorder(new Color(0, 0, 0)));
		this.sensitivityRunPanel.setBounds(panelStartX, sensitivityRunPanelStartY, sensitivityRunPanelWidth,
				sensitivityRunPanelHeight);
		this.sensitivityRunPanel.setLayout(null);

		sensitivityResultsPanelStartY = sensitivityRunPanelStartY + sensitivityRunPanelHeight + outerHeight;
		sensitivityResultsPanelWidth = Math.max((sensitivityMatrixWidth + (2 * outerWidth)) - innerWidth,
				(longLabelWidth + (2 * outerWidth)) - innerWidth);
		sensitivityResultsPanelHeight = sensitivityMatrixHeight + (3 * labelHeight) + (3 * outerHeight) + (innerHeight);
		this.sensitivityResultsPanel = new JPanel();
		this.sensitivityResultsPanel.setBorder(new LineBorder(new Color(0, 0, 0)));
		this.sensitivityResultsPanel.setBounds(panelStartX, sensitivityResultsPanelStartY, sensitivityResultsPanelWidth,
				sensitivityResultsPanelHeight);
		this.sensitivityResultsPanel.setLayout(null);

		int pbWidth = Math.max(buttonWidth, this.sensitivityRunPanel.getWidth() - buttonWidth - (2 * outerWidth)
				- (2 * innerWidth) - plusMinusLabelWidth);

		this.selectExistingAnalysisPanel = new JPanel();
		this.selectExistingAnalysisPanel.setBorder(new LineBorder(new Color(0, 0, 0)));
		this.selectExistingAnalysisPanel.setBounds(panelStartX, selectExistingAnalysisPanelStartY, designPanelWidth,
				sensitivityRunPanelHeight);
		this.selectExistingAnalysisPanel.setLayout(null);

		archiveSensitivityRunPanelStartY = sensitivityResultsPanelStartY + sensitivityResultsPanelHeight + outerHeight;
		archiveSensitivityRunPanelWidth = designVariableVariationRangesPanelWidth;
		archiveSensitivityRunPanelHeight = sensitivityRunPanelHeight;
		this.archiveSensitivityRunPanel = new JPanel();
		this.archiveSensitivityRunPanel.setBorder(new LineBorder(new Color(0, 0, 0)));
		this.archiveSensitivityRunPanel.setBounds(panelStartX, archiveSensitivityRunPanelStartY,
				archiveSensitivityRunPanelWidth, archiveSensitivityRunPanelHeight);
		this.archiveSensitivityRunPanel.setLayout(null);

		this.designPanel.setBounds(this.designPanel.getBounds().x,
				this.designPanel.getBounds().y + this.selectExistingAnalysisPanel.getBounds().height + outerHeight,
				this.designPanel.getBounds().width, this.designPanel.getBounds().height);

		this.designVariableVariationRangesPanel.setBounds(this.designVariableVariationRangesPanel.getBounds().x,
				this.designVariableVariationRangesPanel.getBounds().y
						+ this.selectExistingAnalysisPanel.getBounds().height + outerHeight,
				this.designVariableVariationRangesPanel.getBounds().width,
				this.designVariableVariationRangesPanel.getBounds().height);

		this.targetVariationRangesPanel.setBounds(this.targetVariationRangesPanel.getBounds().x,
				this.targetVariationRangesPanel.getBounds().y + this.selectExistingAnalysisPanel.getBounds().height
						+ outerHeight,
				this.targetVariationRangesPanel.getBounds().width, this.targetVariationRangesPanel.getBounds().height);

		this.sensitivityRunPanel.setBounds(this.sensitivityRunPanel.getBounds().x,
				this.sensitivityRunPanel.getBounds().y + this.selectExistingAnalysisPanel.getBounds().height
						+ outerHeight,
				this.sensitivityRunPanel.getBounds().width, this.sensitivityRunPanel.getBounds().height);

		this.sensitivityResultsPanel.setBounds(this.sensitivityResultsPanel.getBounds().x,
				this.sensitivityResultsPanel.getBounds().y + this.selectExistingAnalysisPanel.getBounds().height
						+ outerHeight,
				this.sensitivityResultsPanel.getBounds().width + (2 * outerWidth) + innerWidth,
				this.sensitivityResultsPanel.getBounds().height);

		this.archiveSensitivityRunPanel.setBounds(this.archiveSensitivityRunPanel.getBounds().x,
				this.archiveSensitivityRunPanel.getBounds().y + +this.selectExistingAnalysisPanel.getBounds().height
						+ outerHeight,
				this.archiveSensitivityRunPanel.getBounds().width, this.archiveSensitivityRunPanel.getBounds().height);

		// the sensitivity control progress bar
		this.sensitivityRunProgressBar = new JProgressBar();
		this.sensitivityRunProgressBar.setStringPainted(true);
		this.sensitivityRunProgressBar.setValue(0);
		this.sensitivityRunProgressBar.setBounds(buttonStartX + buttonWidth + innerWidth, buttonStartY, pbWidth,
				buttonHeight);

		// the select saved analysis combo box
		this.selectAnalysisComboBox = new JComboBox();
		this.selectAnalysisComboBox.setBounds(buttonStartX + buttonWidth + innerWidth, buttonStartY,
				this.selectExistingAnalysisPanel.getBounds().width - buttonWidth - (2 * outerWidth) - (2 * innerWidth)
						- plusMinusLabelWidth,
				buttonHeight);
		this.selectAnalysisComboBox.setEditable(false);
		this.selectAnalysisComboBox.setEnabled(false);
		// ((JLabel) this.selectAnalysisComboBox.getRenderer())
		// .setHorizontalAlignment(SwingConstants.RIGHT);
		this.selectAnalysisComboBox.setMaximumRowCount(TMappingConstants.MAX_STORED_SENSITIVITY_ANALYSES);
		this.selectAnalysisComboBox.addActionListener(new SensitivityComboBoxActionLister());

		// the color scale image
		this.colorScaleImage = new BufferedImage(2 * outerWidth, sensitivityMatrixHeight - labelHeight - outerHeight,
				BufferedImage.TYPE_INT_ARGB);
		for (int j = 0; j < this.colorScaleImage.getHeight(); j++) {
			for (int i = 0; i < this.colorScaleImage.getWidth(); i++) {
				this.colorScaleImage.setRGB(i, j,
						this.getSensitivityColor((1.0 - ((j * 1.0) / this.colorScaleImage.getHeight()))).getRGB());
			}
		}

		this.imageLabel = new JLabel(new ImageIcon(this.colorScaleImage));
		this.imageLabel.setBounds(sensitivityMatrixWidth + outerWidth,
				titleLabelStartY + titleLabelHeight + innerHeight + labelHeight + innerHeight, 2 * outerWidth,
				(sensitivityMatrixHeight - labelHeight - outerHeight));
		this.imageLabel.setBackground(Color.BLUE);

		this.maxColorScaleTitleLabel.setBounds(this.imageLabel.getBounds().x,
				this.imageLabel.getBounds().y - innerHeight - labelHeight, 2 * outerWidth, labelHeight);
		this.minColorScaleTitleLabel.setBounds(this.imageLabel.getBounds().x,
				this.imageLabel.getBounds().y + this.imageLabel.getBounds().height + innerHeight, 2 * outerWidth,
				labelHeight);

		// RUN-AWSA
		this.archiveSensitivityRunButton = new JButton("Run AWSA");
		this.archiveSensitivityRunButton.setBounds(buttonStartX, buttonStartY, (buttonWidth - innerWidth) / 2,
				buttonHeight);
		this.archiveSensitivityRunButton.addActionListener(new SensitivityControlButtonActionLister("RUN-AWSA"));

		// AWSA prefix text field
		this.archiveSensitivityPrefixTextField = new JTextField("", 10);
		this.archiveSensitivityPrefixTextField.setEditable(true);
		this.archiveSensitivityPrefixTextField.setBounds(buttonStartX + ((buttonWidth - innerWidth) / 2) + innerWidth,
				buttonStartY, (buttonWidth - innerWidth) / 2, buttonHeight);
		this.archiveSensitivityPrefixTextField.setFont(new Font("Tahoma", Font.PLAIN, 11));
		this.archiveSensitivityPrefixTextField.setText("T00_");

		// the AWSA progress bar
		this.archiveSensitivityRunProgressBar = new JProgressBar();
		this.archiveSensitivityRunProgressBar.setStringPainted(true);
		this.archiveSensitivityRunProgressBar.setValue(0);
		this.archiveSensitivityRunProgressBar.setBounds(buttonStartX + buttonWidth + innerWidth, buttonStartY, pbWidth,
				buttonHeight);

		// scroll bars
		this.setPreferredSize(new Dimension(this.designPanel.getWidth() + (2 * outerWidth),
				this.selectExistingAnalysisPanel.getHeight() + this.designPanel.getHeight()
						+ this.designVariableVariationRangesPanel.getHeight() + this.sensitivityRunPanel.getHeight()
						+ this.sensitivityResultsPanel.getHeight() + this.archiveSensitivityRunPanel.getHeight()
						+ (6 * outerHeight)));

	}

	public void refresh() {

		this.add(this.selectExistingAnalysisPanel);
		this.selectExistingAnalysisPanel.removeAll();
		this.selectExistingAnalysisPanel.validate();

		this.selectExistingAnalysisPanel.add(this.selectExistingAnalysisTitleLabel);

		this.selectAnalysisComboBox
				.setEnabled(this.parentMappingManager.getSensitivityRunState().equals(TThreadState.IDLE));
		if (this.parentMappingManager.getNrOfSavedAnalyses() != 0) {
			String[] comboValues = new String[this.parentMappingManager.getNrOfSavedAnalyses()];
			comboValues = this.parentMappingManager.getSavedAnalysisNames();
			DefaultComboBoxModel dcbm = new DefaultComboBoxModel(comboValues);
			this.selectAnalysisComboBox.setModel(dcbm);
			if (this.parentMappingManager.isUsingSavedAnalysis()) {
				this.selectAnalysisComboBox
						.setSelectedIndex(this.parentMappingManager.getCurrentSensitivityAnalysisDisplayIndex());
			} else {
				this.selectAnalysisComboBox.setSelectedIndex(-1);
			}
		}

		this.selectExistingAnalysisPanel.add(this.selectAnalysisComboBox);

		this.selectExistingAnalysisPanel.validate();
		this.selectExistingAnalysisPanel.repaint();

		this.add(this.designPanel);
		this.designPanel.removeAll();
		this.designPanel.validate();

		Individual chosenInd = this.parentMappingManager.getcurrentBaseIndividual();

		Map<String, Double> variableValues = new HashMap<String, Double>();
		Map<String, Double> targetValues = new HashMap<String, Double>();
		if (chosenInd != null) {
			for (VariableDesc vDesc : this.parentMappingManager.getProblem().getVariedVariables()) {
				variableValues.put(vDesc.getName(), chosenInd.getVariable(vDesc.getName()));
			}
			for (ResultDesc rDesc : this.parentMappingManager.getProblem().getResults()) {
				if (rDesc.isObjective() || rDesc.isConstraint()) {
					targetValues.put(rDesc.getName(), chosenInd.getResult(rDesc.getName()));
				}
			}
		}

		this.designPanel.add(this.selectedDesignTitleLabel);
		this.designPanel.add(this.originalPerformanceTitleLabel);
		for (VariableDesc vDesc : this.parentMappingManager.getProblem().getVariedVariables()) {

			if (chosenInd != null) {
				this.designVariableFields.get(vDesc.getName())
						.setText(String.format("%.8f", variableValues.get(vDesc.getName())));
			} else {
				this.designVariableFields.get(vDesc.getName()).setText("-");
			}

			this.designPanel.add(this.designVariableLabels.get(vDesc.getName()));
			this.designPanel.add(this.designVariableFields.get(vDesc.getName()));
		}
		for (String targetName : this.parentMappingManager.getSensitivityTargetNames()) {
			if (chosenInd != null) {
				this.targetFields.get(targetName).setText(String.format("%.8f", targetValues.get(targetName)));
			} else {
				this.targetFields.get(targetName).setText("-");
			}

			this.designPanel.add(this.targetLabels.get(targetName));
			this.designPanel.add(this.targetFields.get(targetName));

		}

		this.designPanel.validate();
		this.designPanel.repaint();

		this.add(this.designVariableVariationRangesPanel);
		this.designVariableVariationRangesPanel.removeAll();
		this.designVariableVariationRangesPanel.validate();

		this.designVariableVariationRangesPanel.add(this.variableRangesTitleLabel);
		this.designVariableVariationRangesPanel.add(this.plusVariableRangesTitleLabel);
		this.designVariableVariationRangesPanel.add(this.minusVariableRangesTitleLabel);

		for (VariableDesc vDesc : this.parentMappingManager.getProblem().getVariedVariables()) {

			if (chosenInd != null) {
				this.plusVariableRangesTextFields.get(vDesc.getName()).setText(
						String.format("%.8f", this.parentMappingManager.getPlusVariableRange(vDesc.getName())));
				this.minusVariableRangesTextFields.get(vDesc.getName()).setText(
						String.format("%.8f", this.parentMappingManager.getMinusVariableRange(vDesc.getName())));
				this.plusVariableRangesTextFields.get(vDesc.getName())
						.setEnabled(this.parentMappingManager.getSensitivityRunState().equals(TThreadState.IDLE)
								&& !this.parentMappingManager.isUsingSavedAnalysis());
				this.minusVariableRangesTextFields.get(vDesc.getName())
						.setEnabled(this.parentMappingManager.getSensitivityRunState().equals(TThreadState.IDLE)
								&& !this.parentMappingManager.isUsingSavedAnalysis());
			} else {
				this.plusVariableRangesTextFields.get(vDesc.getName()).setText("-");
				this.minusVariableRangesTextFields.get(vDesc.getName()).setText("-");
				this.plusVariableRangesTextFields.get(vDesc.getName()).setEnabled(false);
				this.minusVariableRangesTextFields.get(vDesc.getName()).setEnabled(false);
			}

			this.designVariableVariationRangesPanel.add(this.plusVariableRangesTextFields.get(vDesc.getName()));
			this.designVariableVariationRangesPanel.add(this.minusVariableRangesTextFields.get(vDesc.getName()));
		}

		this.designVariableVariationRangesPanel.validate();
		this.designVariableVariationRangesPanel.repaint();

		this.add(this.targetVariationRangesPanel);
		this.targetVariationRangesPanel.removeAll();
		this.targetVariationRangesPanel.validate();

		this.targetVariationRangesPanel.add(this.targetRangesTitleLabel);
		this.targetVariationRangesPanel.add(this.plusTargetTitleLabel);
		this.targetVariationRangesPanel.add(this.minusTargetTitleLabel);

		for (String targetName : this.parentMappingManager.getSensitivityTargetNames()) {

			String valueToDisplay = this.parentMappingManager
					.getGlobalSensitivityAbsolutePlusValueToDisplay(targetName);

			this.plusTargetRangesTextFields.get(targetName).setText(valueToDisplay);

			if (!valueToDisplay.equals("-/-")) {
				this.plusTargetRangesTextFields.get(targetName).setBackground(SystemColor.controlHighlight);
			} else {
				this.plusTargetRangesTextFields.get(targetName).setBackground(SystemColor.control);
			}

			valueToDisplay = this.parentMappingManager.getGlobalSensitivityAbsoluteMinValueToDisplay(targetName);

			this.minusTargetRangesTextFields.get(targetName).setText(valueToDisplay);

			if (!valueToDisplay.equals("-/-")) {
				this.minusTargetRangesTextFields.get(targetName).setBackground(SystemColor.controlHighlight);
			} else {
				this.minusTargetRangesTextFields.get(targetName).setBackground(SystemColor.control);
			}

			this.targetVariationRangesPanel.add(this.plusTargetRangesTextFields.get(targetName));
			this.targetVariationRangesPanel.add(this.minusTargetRangesTextFields.get(targetName));

		}

		this.targetVariationRangesPanel.validate();
		this.targetVariationRangesPanel.repaint();

		this.add(this.sensitivityRunPanel);
		this.sensitivityRunPanel.removeAll();
		this.sensitivityRunPanel.validate();

		boolean isWaitingForFEResults = false;
		if (this.parentMappingManager.isUsingSavedAnalysis()) {
			this.sensitivityRunControlButtons.get("RUN-MAP").setText("Re-evaluate using current mappings");
			this.sensitivityRunControlButtons.get("SAVE").setText("Save re-evaluation results");
			this.sensitivityRunControlButtons.get("DISCARD").setText("Discard re-evaluation results");

			this.sensitivityRunControlButtons.get("RUN-FE").setEnabled(false);
			this.sensitivityRunControlButtons.get("CHECK-FE").setEnabled(false);
			if (this.parentMappingManager.isUsingSAWithFEResults()
					&& !this.parentMappingManager.isUsingSAWaitingForFEResults()) {
				this.sensitivityRunControlButtons.get("RUN-FE").setText("FE-analysis completed");
			} else if (this.parentMappingManager.isUsingSAWaitingForFEResults()) {
				// this.sensitivityRunControlButtons.get("RUN-FE").setText(
				// "FE-analysis in progress");
				isWaitingForFEResults = true;
				this.sensitivityRunControlButtons.get("CHECK-FE").setEnabled(true);
			} else {
				this.sensitivityRunControlButtons.get("RUN-FE").setText("Run FE-based analysis");
				this.sensitivityRunControlButtons.get("RUN-FE").setEnabled(true);
			}

		} else {
			this.sensitivityRunControlButtons.get("RUN-MAP").setText("Run new mapping-based analysis");
			this.sensitivityRunControlButtons.get("SAVE").setText("Save mapping-based analysis");
			this.sensitivityRunControlButtons.get("DISCARD").setText("Discard mapping-based analysis");
		}

		if (this.parentMappingManager.getSensitivityRunState().equals(TThreadState.IDLE)) {
			this.sensitivityRunPanel.add(this.sensitivityRunControlButtons.get("RUN-MAP"));
			if (chosenInd != null) {
				this.sensitivityRunControlButtons.get("RUN-MAP")
						.setEnabled(this.parentMappingManager.areAllMappingHoldersExceptErrorsClsReady());
				if (this.parentMappingManager.isUsingSavedAnalysis() && !(isWaitingForFEResults)) {
					this.sensitivityRunPanel.add(this.sensitivityRunControlButtons.get("RUN-FE"));
				}
				if (this.parentMappingManager.isUsingSavedAnalysis() && (isWaitingForFEResults)) {
					this.sensitivityRunControlButtons.get("CHECK-FE")
							.setText(this.parentMappingManager.getFESensitivityProgress());
					this.sensitivityRunPanel.add(this.sensitivityRunControlButtons.get("CHECK-FE"));
				}
			} else {
				this.sensitivityRunControlButtons.get("RUN-MAP").setEnabled(false);
			}
		}
		if (this.parentMappingManager.getSensitivityRunState().equals(TThreadState.RUNNING)) {
			this.sensitivityRunPanel.add(this.sensitivityRunControlButtons.get("STOP"));
			this.sensitivityRunProgressBar.setValue(this.parentMappingManager.getSensitivityAnalysisProgressValue());
			this.sensitivityRunPanel.add(this.sensitivityRunProgressBar);
		}
		if (this.parentMappingManager.getSensitivityRunState().equals(TThreadState.READY)) {

			this.sensitivityRunControlButtons.get("SAVE").setEnabled(true);
			this.sensitivityRunPanel.add(this.sensitivityRunControlButtons.get("SAVE"));

			this.sensitivityRunPanel.add(this.sensitivityRunControlButtons.get("DISCARD"));
		}

		this.sensitivityRunPanel.validate();
		this.sensitivityRunPanel.repaint();

		this.add(this.sensitivityResultsPanel);
		this.sensitivityResultsPanel.removeAll();
		this.sensitivityResultsPanel.validate();

		this.sensitivityResultsPanel.add(this.imageLabel);

		boolean labelsAdded = false;
		for (VariableDesc vDesc : this.parentMappingManager.getProblem().getVariedVariables()) {
			for (String targetName : this.parentMappingManager.getSensitivityTargetNames()) {
				this.localSensitivityTextFields.get(vDesc.getName()).get(targetName).setText(
						this.parentMappingManager.getLocalSensitivityValueToDisplay(vDesc.getName(), targetName));

				double colorValue = this.parentMappingManager.getNormalizedSensitivity(vDesc.getName(), targetName);

				if (this.parentMappingManager.isUsingSAWithMappingResults()) {
					this.useMappingColoring.setEnabled(true);
				} else {
					this.useMappingColoring.setEnabled(false);
				}

				if (this.parentMappingManager.isUsingSAWithFEResults()) {
					this.useFEColoring.setEnabled(true);
				} else {
					this.useFEColoring.setEnabled(false);
				}

				this.sensitivityResultsPanel.add(this.useMappingColoring);
				this.sensitivityResultsPanel.add(this.useFEColoring);

				if (colorValue > 0) {
					this.localSensitivityTextFields.get(vDesc.getName()).get(targetName)
							.setBackground(this.getSensitivityColor(colorValue));
				} else {
					this.localSensitivityTextFields.get(vDesc.getName()).get(targetName)
							.setBackground(SystemColor.control);
				}

				this.sensitivityResultsPanel.add(this.localSensitivityTextFields.get(vDesc.getName()).get(targetName));

				if (!labelsAdded) {
					this.sensitivityResultsPanel.add(this.localSensitivityTargetLabels.get(targetName));

					String valueToDisplay = this.parentMappingManager
							.getGlobalSensitivityRelativeValueToDisplay(targetName);

					this.globalSensitivityTextFields.get(targetName).setText(valueToDisplay);

					if (!valueToDisplay.equals("-/-")) {
						this.globalSensitivityTextFields.get(targetName).setBackground(SystemColor.controlHighlight);
					} else {
						this.globalSensitivityTextFields.get(targetName).setBackground(SystemColor.control);
					}

					this.sensitivityResultsPanel.add(this.globalSensitivityTextFields.get(targetName));
				}
			}

			String originalName = vDesc.getName().substring(vDesc.getName().lastIndexOf(".") + 1);
			this.localSensitivityDesignVariableLabels.get(vDesc.getName()).setText(originalName);
			if (chosenInd != null) {
				if (chosenInd.getVariable(vDesc.getName()) == 0.0) {
					this.localSensitivityDesignVariableLabels.get(vDesc.getName()).setText(originalName + "*");
				}

			}

			this.sensitivityResultsPanel.add(this.localSensitivityDesignVariableLabels.get(vDesc.getName()));
			labelsAdded = true;
		}

		this.sensitivityResultsPanel.add(this.localSensitivityResultsTitleLabel);
		this.sensitivityResultsPanel.add(this.globalSensitivityResultsTitleLabel);

		this.sensitivityResultsPanel.add(this.maxColorScaleTitleLabel);

		this.sensitivityResultsPanel.add(this.minColorScaleTitleLabel);

		this.sensitivityResultsPanel.validate();
		this.sensitivityResultsPanel.repaint();

		this.add(this.archiveSensitivityRunPanel);
		this.archiveSensitivityRunPanel.removeAll();
		this.archiveSensitivityRunPanel.validate();

		this.archiveSensitivityRunButton.setEnabled(this.parentMappingManager.areAllMappingHoldersExceptErrorsClsReady()
				&& !this.parentMappingManager.isPerformingAWSA());
		this.archiveSensitivityPrefixTextField
				.setEnabled(this.parentMappingManager.areAllMappingHoldersExceptErrorsClsReady()
						&& !this.parentMappingManager.isPerformingAWSA());
		this.archiveSensitivityRunProgressBar.setValue(this.parentMappingManager.getAWSAsProgressValue());
		this.archiveSensitivityRunProgressBar
				.setEnabled(this.parentMappingManager.areAllMappingHoldersExceptErrorsClsReady());

		this.archiveSensitivityRunPanel.add(this.archiveSensitivityRunButton);
		this.archiveSensitivityRunPanel.add(this.archiveSensitivityPrefixTextField);
		this.archiveSensitivityRunPanel.add(this.archiveSensitivityRunProgressBar);

		this.archiveSensitivityRunPanel.validate();
		this.archiveSensitivityRunPanel.repaint();

		this.validate();
		this.repaint();
	}

	public Color getSensitivityColor(double sensitivity) {
		int R = (int) (255.0 * sensitivity);
		int G = (int) (255.0 * (1.0 - sensitivity));
		int B = 0;

		// float HSB[] = new float[3];
		// Color.RGBtoHSB(R, G, B, HSB);

		return new Color(R, G, B, 255);
	}
}
