from docutils.core import publish_string
import os
import sys

def main(basepath, storepath, cut_str):
    
    basepath = os.path.abspath(basepath)
    
    # get all .py-files
    file_ok = True
    if os.path.isdir(basepath):
        files = [x for x in os.listdir(basepath) if x.endswith('.py')]
        storepath = os.path.join(basepath, storepath)
    elif os.path.isfile(basepath):
        files = [os.path.split(basepath)[-1]]
        basepath = os.path.split(basepath)[0]
        storepath = os.path.join(basepath, storepath)
    else:
        print(f'ERROR: basepath {basepath} is neither a valid directory nor a valid file!')
        file_ok = False
        
    if file_ok:
        # create outputdirecotry if not here
        if not os.path.isdir(storepath):
            os.mkdir(storepath)

        # create html for all .py files
        for fname in files:
            fullpath = os.path.join(basepath, fname)
            with open(fullpath,'r') as fh:
                lines = ''.join(fh.readlines())
                docstr_found = False

                # fook for cur_str
                if cut_str in lines:
                    _ = lines.split(cut_str)[1]

                    # look for first comment block
                    if '"""' in _:
                        docstr = _.split('"""')[1]
                        
                        # remove trailing white space
                        spaces = None
                        docstr_lines = docstr.split('\n')
                        for idx,line in enumerate(docstr_lines):
                            if (len(line) > 0) & (spaces == None):
                                # only set from first line!
                                spaces = len(line) - len(line.lstrip())
                            if (spaces is not None) and (len(line) > spaces):
                                docstr_lines[idx] = line[spaces:]
                        
                        # automatically add some header ...
                        docstr_lines.insert(0,fname)
                        docstr_lines.insert(1,'='*len(fname))
                        
                        docstr = '\n'.join(docstr_lines)
                        html_str = publish_string(docstr, writer_name='html')
                        docstr_found = True

            if docstr_found:
                with open(os.path.join(storepath,fname.rsplit('.py',1)[0]+'.html'),'wb') as fh:
                    fh.write(html_str)
                print(f"Documentation string written to {fh.name}")
            else:
                print(f"ERROR: No documentation string found for {os.path.split(fullpath)[-1]}")
            
if __name__ == '__main__':
    """
    This script can be used to create a short documentation of SyMSpace macro scripts.

    Running this script on the command line, it takes three *optional* arguments:

        `python createScriptShortInfo.py <base_path> <store_path> <cut_str>`
        
    **<base_path>**
    
        Defines a directory or a single script file. If a directory is defined, all .py files
        of the folder are processed.
        
        *delault*: `<MAGOPT_HOME>/System/Python/scripts/`
    
    **<store_path>**
    
        Defines the path relative to `<base_path>`, where the created html files are stored.
        If the directory does not exist, it will be created.
        
        *delault*: `html`
    
    **<cut_str>**
    
        The first comment block, encosed by tripple quotes, after this string will be processed
        as documentation string.
        
        *delault*: `if __name__ == '__main__':\n`
        
        Thus, by default, the first comment block just after `if __name__ == '__main__':` will be
        used.
        
    To create this document, run:
    
        `python createScriptShortInfo.py ./createScriptShortInfo.py`
    
    """
    
    args_ok = True
    
    # base params
    basepath = os.path.join(os.environ['MAGOPT_HOME'],'System','Python','scripts')
    cut_str = "if __name__ == '__main__':\n"
    storepath = 'html'
    
    if len(sys.argv) > 1:
        basepath = os.path.abspath(sys.argv[1])
    if len(sys.argv) > 2:
        storepath = os.path(sys.argv[2])
        print(storepath)
    if len(sys.argv) > 3:
        cut_str = sys.argv[3]
        print(str(sys.argv[3]))
    if len(sys.argv) > 4:
        print("ERROR: Too much command line parameters given!")
        args_ok = False
    
    if args_ok:
        main(basepath, storepath, cut_str)
    
                