#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 10062 $
$LastChangedDate:: 2020-11-20 13:22:02 +0100#$

This file is part of SyMSpace.

Copyright (c) 2019, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""

from symspace.console.script_utils import ScriptUtils
from symspace.wrapper.symwrapper import SymContainer, SymMaterial, SymFunction
from symspace.utils.comparison import ContainerComparison


def main(parameter1: SymContainer, parameter2: SymContainer):

    if parameter2 is None:
        raise Exception('Set reference container as CurObject (Ctrl->B)\n\n'
                        'The currently selected container in the symspace-tree will be used as second container')

    for param in [parameter1, parameter2]:
        if not isinstance(param, (SymContainer, SymMaterial, SymFunction)):
            raise Exception('Parameter must be a subclass of SymContainer!')

    cmp = ContainerComparison.fromSymContainer(parameter1, parameter2)
    cmp.showHTMLTable()


if __name__ == '__main__':
    """This script compares the results of two SymContainers

        Set reference container as CurObject (Ctrl->B)

        The currently selected container in the symspace-tree will be used as second container
    """
    _params = locals()
    _parameter1 = ScriptUtils.getParameter(_params)
    _parameter2 = ScriptUtils.getParameter(_params, manual_assigned=True)

    ScriptUtils.run(main, _parameter1, _parameter2)
