#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 10062 $
$LastChangedDate:: 2020-11-20 13:22:02 +0100#$

This file is part of SyMSpace.

Copyright (c) 2018, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""
import os
from lcm.utils.serialization import Serializable
from symspace.wrapper.symwrapper import SymProject, SymField, SymParameter
from symspace.generic_gui_qt import GUIFormThread
from symspace.console.script_utils import ScriptUtils


def main(project: SymProject, parameter: SymParameter):

    if isinstance(parameter, SymField):
        default_path = os.path.join(project.getDirectory(), parameter.getName() + '.mat')

        form_title = 'Load data from file'
        form_elements = [('Select file', 'browse_file', default_path,
                          'Pickle Files (*.p *.pickle);;MAT-files (*.mat);;CBOR-files (*.cbor)')]
        form = GUIFormThread(form_title, form_elements)

        form.start()
        if form.accept:
            path = os.path.join(form.result['Select file'])
            obj = Serializable.loadFile(path)
            parameter.setData(obj)

            print('Import finished')
        else:
            print('Cancelled by user')
    else:
        raise Exception('Only Fields are supported for import!')


if __name__ == '__main__':
    """
    This script loads a parameter from Pickle(.p, .pickle), CBOR (*.cbor) or MAT-files (*.mat)
    
    This script needs to be executed in the SyMSpace Python Console.
    """

    _params = locals()
    _project = ScriptUtils.getProject(_params)
    _parameter = ScriptUtils.getParameter(_params)
    if _project is not None:
        ScriptUtils.run(main, _project, _parameter)
    else:
        ScriptUtils.showNoProjectWarning()