#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.5

$LastChangedRevision: 10062 $
$LastChangedDate:: 2020-11-20 13:22:02 +0100#$

This file is part of SyMSpace.

Copyright (c) 2017, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""

import subprocess
import os
import shutil
import time
from symspace.generic_gui_qt import GUIFormThread
from symspace.osutils.pathutils import Paths
from symspace.console.script_utils import ScriptUtils
from symspace.wrapper.symoptwrapper import SymProject


def main(project: SymProject):
    if project is None:
        startdir = Paths.getSyMSpaceRoot()
    else:
        startdir = project.getDirectory() + '\\'
    form_title = 'Open Jupyter Notebook'
    form_elements = [('Select a notebook file', 'browse_file', startdir, 'Project files (*.ipynb)'),
                     ('Create new if it does not exist', 'checkbox', True),
                     ('Open in Jupyter Lab', 'checkbox', False)
                     ]
    form = GUIFormThread(form_title, form_elements)
    form.start()

    if form.accept:
        ipynb = form.result['Select a notebook file']
        openlab = form.result['Open in Jupyter Lab']
        if not ipynb.endswith('.ipynb'):
            ipynb += '.ipynb'
        create_new = form.result['Create new if it does not exist']
        if not os.path.exists(ipynb):
            if create_new:
                template = os.path.join(Paths.getSyMSpaceRoot(), 'System', 'Python', 'resources', 'NewJupyterNotebook',
                                        'template.ipynb')
                shutil.copyfile(template, ipynb)
                time.sleep(3)
            else:
                raise Exception('File {} does not exist'.format(ipynb))

        print('jupyter notebook "{}"'.format(ipynb))
        if not openlab:
            subprocess.Popen('jupyter notebook "{}"'.format(ipynb))
        else:
            subprocess.Popen('jupyter lab "{}"'.format(ipynb))

    else:
        print('Cancelled by user')


if __name__ == '__main__':
    """
    This script opens an existing Jupyter Notebook and is able to create a new one, if it does not already exist
    """

    _params = locals()
    _project = ScriptUtils.getProject(_params)
    ScriptUtils.run(main, _project)
