# !/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 10056 $
$LastChangedDate:: 2020-11-11 13:09:26 +0100#$

This file is part of SyMSpace.

Copyright (c) 2020, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""
import os
import pipreqs.pipreqs
from docopt import docopt


class PipUtil:

    @staticmethod
    def createRequirementsFile(pkg_path: str, target_file: str = None, overwrite: bool = True,
                               no_versions: bool = False, only_local: bool = True,
                               dry_run: bool = False) -> str:
        """
        Create a requirements file from a source package

        Parameters
        ----------
        pkg_path: str
            Path of the source package to analyze
        target_file: str
            Save the list of requirements in the given file
        overwrite: bool
            Overwrite the existing requirements file
        no_versions: bool
            Do not collect package versions
        only_local:
            Use ONLY local package info instead of querying PyPI
        dry_run: bool
            Only print the requirements, but do not write a file

        Returns
        -------
        req_file : str
            Requirements file path
        """
        if target_file is None:
            req_file = os.path.join(pkg_path, 'requirements.txt')
        else:
            req_file = target_file

        argv = ['--savepath', req_file]

        if dry_run:
            argv.append('--print')
        if overwrite:
            argv.append('--force')
        if no_versions:
            argv.append('--no-pin')
        if only_local:
            argv.append('--use-local')
        argv.append(pkg_path)
        args = docopt(pipreqs.pipreqs.__doc__, argv=argv)
        pipreqs.pipreqs.init(args)

        if not dry_run and not os.path.isfile(req_file):
            raise FileNotFoundError('File does not exist "{}"'.format(req_file))
        return req_file

    @staticmethod
    def mergeRequirementFiles(files: list):
        raise NotImplementedError('PipUtils.mergeRequirementFiles is not implemented yet!')
