# !/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 10105 $
$LastChangedDate:: 2021-02-08 10:10:23 +0100#$

This file is part of SyMSpace.

Copyright (c) 2020, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""
import logging


class LoggerClass:
    default_log_level = logging.WARNING

    def __init__(self, loglevel=None):
        self.logger = self._setLogger(loglevel)

    def errorFromException(self, ex: Exception):
        self.logger.error('{}: {}'.format(ex.__class__.__name__, ex))

    @classmethod
    def getLoggerName(cls) -> str:
        name = cls.__name__  # class name
        # name = cls.__module__ # module path
        return name

    def setLogLevel(self, loglevel):
        """
        Set the logging level of this logger.

        Parameters
        ----------
        loglevel: int or str
        """
        if isinstance(loglevel, str):
            loglevel = loglevel.upper()
        self.logger.setLevel(loglevel)
        for handler in self.logger.handlers:
            handler.setLevel(loglevel)

    @classmethod
    def _setLogger(cls, loglevel=None):
        if loglevel is None:
            loglevel = cls.default_log_level
        elif isinstance(loglevel, str):
            loglevel = loglevel.upper()

        name = cls.getLoggerName()

        if name in logging.Logger.manager.loggerDict:
            '''avoid creating a handlers for logger twice'''
            logger = logging.getLogger(name)
            logger.setLevel(loglevel)
        else:
            logger = logging.getLogger(name)
            logger.setLevel(loglevel)

            sh = logging.StreamHandler()
            formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s', "%H:%M:%S")
            sh.setFormatter(formatter)
            sh.setLevel(loglevel)
            logger.addHandler(sh)
            logger.debug('Logger created!')
        return logger
