#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 9967 $
$LastChangedDate:: 2020-08-24 11:17:15 +0200#$

This file is part of SyMSpace.

Copyright (c) 2019, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""
import os
import optparse
from symspace.osutils.pathutils import Paths
from symspace.osutils.pathutils import CMDParser

try:
    from java.io import File
    from at.lcm.bu21.magopt.structure import ParamMaterial
    from magopt.structure.database import FileConnection
except Exception as e:
    print('Unable to load Java Libraries!\n{}'.format(e))


class MaterialDataBase:

    @classmethod
    def cmdToFile(cls):
        def_file_path = os.path.join(Paths.getSyMSpaceRoot(), 'material.mat')
        parser = optparse.OptionParser("usage: %prog [options]")
        parser.add_option("--filePath", dest="file_path",
                          default=def_file_path, type="str",
                          help="File path for material database export")

        args = CMDParser.jepSysArgs()
        (options, args) = parser.parse_args(args)

        file_path = CMDParser.jepArgPath(options.file_path)

        cls.toFile(file_path)

    @staticmethod
    def toFile(file_path):
        if not file_path.endswith('.mat'):
            raise Exception('File has to be a ".mat" file!')
        FileConnection.createFile(File(file_path), ParamMaterial.getConnection())
        print('Material DB exported to: {}'.format(file_path))


if __name__ == '__main__':
    MaterialDataBase.toFile('material.mat')
