# -*- coding: utf-8 -*-
"""
Created on Thu Okt 06 16:39:02 2017

@author: mkitzberger
"""

import unittest
from symspace.osutils.processutils import getInterpreter
from symspace.wrapper.symwrapper import SymProject, SymField, SymFormula, SymContainer, SymParameter,\
    SymMatlabFunction, SymPythonFunction, SymJavaFunction, SymMaterial, SymFile, SymDirectory


@unittest.skipIf(getInterpreter() == 'python', 'needs jep interpreter to run')
class TestSymParam(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.CurProject1 = SymProject('test', 'test', 'test')
        cls.param1 = SymField('param1Name', 'param1Desc')
        cls.param2 = SymField('param2Name', 'param2Desc')
        cls.param3 = SymField('param3Name', 'param3Desc')
        cls.param4 = SymField('param4Name', 'param4Desc')
        cls.param5 = SymField('param5Name', 'param5Desc')
        cls.param6 = SymField('param6Name', 'param6Desc')
        cls.CurProject1.addParam(cls.param1)
        cls.CurProject1.addParam(cls.param2)
        cls.CurProject1.addParam(cls.param3)
        cls.CurProject1.addParam(cls.param4)
        cls.CurProject1.addParam(cls.param5)
        cls.CurProject1.addParam(cls.param6)

        cls.CurProject2 = SymProject('test', 'test', 'test')
        cls.formula = SymFormula('test', 'test')
        cls.CurProject2.addParam(cls.param1)
        cls.CurProject2.addParam(cls.param2)
        cls.CurProject2.addParam(cls.param3)
        cls.CurProject2.addParam(cls.formula)
        cls.container1 = SymContainer('container1Name', 'container1Desc')
        cls.containerSubParam = SymField('param1Name')
        cls.container1.addParam(cls.containerSubParam)
        cls.CurProject2.addParam(cls.container1)

    def test_javaObj(self):
        self.assertEqual(self.param1.javaObj, self.param1.getJava())
        paramJava = self.param1.javaObj
        param = SymParameter.convertToPython(paramJava)
        self.assertIsInstance(param, SymField)
        self.assertEqual(param.javaObj, paramJava)

        self.assertEqual(self.CurProject1.javaObj, self.CurProject1.getJava())
        paramJava = self.CurProject1.javaObj
        param = SymParameter.convertToPython(paramJava)
        self.assertIsInstance(param, SymProject)
        self.assertEqual(param.javaObj, paramJava)

    def test_convertToPython(self):
        project = SymProject('test', 'test', 'test')
        material = SymMaterial('test', 'test')
        matlabFun = SymMatlabFunction('test', 'test')
        pythonFun = SymPythonFunction('test', 'test')
        javaFun = SymJavaFunction('test', 'test')
        container = SymContainer('test', 'test')
        formula = SymFormula('test', 'test')
        file = SymFile('test', 'test')
        directory = SymDirectory('test', 'test')

        self.assertIsInstance(SymParameter.convertToPython(project.javaObj), SymProject)
        self.assertEqual(SymParameter.convertToPython(project.javaObj).javaObj, project.javaObj)

        self.assertIsInstance(SymParameter.convertToPython(material.javaObj), SymMaterial)
        self.assertEqual(SymParameter.convertToPython(material.javaObj).javaObj, material.javaObj)

        self.assertIsInstance(SymParameter.convertToPython(matlabFun.javaObj), SymMatlabFunction)
        self.assertEqual(SymParameter.convertToPython(matlabFun.javaObj).javaObj, matlabFun.javaObj)

        self.assertIsInstance(SymParameter.convertToPython(pythonFun.javaObj), SymPythonFunction)
        self.assertEqual(SymParameter.convertToPython(pythonFun.javaObj).javaObj, pythonFun.javaObj)

        self.assertIsInstance(SymParameter.convertToPython(javaFun.javaObj), SymJavaFunction)
        self.assertEqual(SymParameter.convertToPython(javaFun.javaObj).javaObj, javaFun.javaObj)

        self.assertIsInstance(SymParameter.convertToPython(container.javaObj), SymContainer)
        self.assertEqual(SymParameter.convertToPython(container.javaObj).javaObj, container.javaObj)

        self.assertIsInstance(SymParameter.convertToPython(formula.javaObj), SymFormula)
        self.assertEqual(SymParameter.convertToPython(project.javaObj).javaObj, project.javaObj)

        self.assertIsInstance(SymParameter.convertToPython(file.javaObj), SymFile)
        self.assertEqual(SymParameter.convertToPython(file.javaObj).javaObj, file.javaObj)

        self.assertIsInstance(SymParameter.convertToPython(directory.javaObj), SymDirectory)
        self.assertEqual(SymParameter.convertToPython(directory.javaObj).javaObj, directory.javaObj)

    def test_removeParam(self):
        self.param1.removeParam()
        self.param2.removeParam()
        self.param3.removeParam()
        self.assertEqual(self.CurProject1.getParam(), [self.param4, self.param5, self.param6])

    def test_equality(self):
        param_conv = SymParameter.convertToPython(self.param1.getJava())
        self.assertTrue(param_conv == self.param1)
        param_conv.javaObj = self.param2.javaObj
        self.assertFalse(param_conv == self.param1)

    def testCloneParam(self):
        cloned_param = self.param1.cloneParam()
        self.assertTrue(cloned_param, self.param1)
        param1 = SymField('test1', 'test1')
        param2 = SymField('test2', 'test2')
        self.assertFalse(param1 == param2)
        param2.cloneParam(param1)
        self.assertTrue(param1 == param2)

    def test_getName(self):
        self.assertEqual(self.param1.getName(), 'param1Name')
        self.assertEqual(self.param2.getName(), 'param2Name')

    @unittest.skip
    def test_getDynamicOutputs(self):
        self.fail()

    @unittest.skip
    def test_getDynamicParams(self):
        self.fail()

    @unittest.skip
    def test_getAnalysisParams(self):
        self.fail()

    @unittest.skip
    def test_getObject(self):
        firstOccurence = self.CurProject2.getObject('param1Name')
        self.assertEqual(firstOccurence, self.param1)

    @unittest.skip
    def test_getUniqueObject(self):
        self.fail()

    @unittest.skip
    def test_getObjectPath(self):
        self.fail()

    @unittest.skip
    def test_getParamByIdentifier(self):
        self.fail()

    @unittest.skip
    def test_getParamByPid(self):
        self.fail()

    @unittest.skip
    def test_getParamByUniqueIdentifier(self):
        self.fail()

    @unittest.skip
    def test_getParameters(self):
        self.fail()

    def test_getParent(self):
        self.assertEqual(self.container1.getParent(), self.CurProject2)
        self.assertEqual(self.containerSubParam.getParent(), self.container1)

    @unittest.skip
    def test_getPId(self):
        self.fail()

    @unittest.skip
    def test_getDescription(self):
        self.fail()

    @unittest.skip
    def test_getModified(self):
        self.fail()

    @unittest.skip
    def test_getUncertain(self):
        self.fail()

    @unittest.skip
    def test_getPrint(self):
        self.fail()

    @unittest.skip
    def test_getReadOnly(self):
        self.fail()

    @unittest.skip
    def test_getUsed(self):
        self.fail()

    @unittest.skip
    def test_getUsedBy(self):
        self.fail()

    @unittest.skip
    def test_getSetBy(self):
        self.fail()

    @unittest.skip
    def test_getUsedInputParameter(self):
        self.fail()

    @unittest.skip
    def test_getError(self):
        self.fail()

    @unittest.skip
    def test_getIndex(self):
        self.fail()

    @unittest.skip
    def test_getConstant(self):
        self.fail()

    def test_getRoot(self):
        self.assertEqual(self.container1.getRoot(), self.CurProject2)
        self.assertEqual(self.param1.getRoot(), self.CurProject2)
        self.assertEqual(self.containerSubParam.getRoot(), self.CurProject2)

    @unittest.skip
    def test_getIdentifier(self):
        self.fail()

    @unittest.skip
    def test_getLength(self):
        self.fail()

    @unittest.skip
    def test_getData(self):
        self.fail()

    @unittest.skip
    def test_getJavaData(self):
        self.fail()

    @unittest.skip
    def test_getDataString(self):
        self.fail()

    @unittest.skip
    def test_getSetBy(self):
        self.fail()

    @unittest.skip
    def test_getComment(self):
        self.fail()

    @unittest.skip
    def test_getJava(self):
        self.fail()

    @unittest.skip
    def test_setName(self):
        self.fail()

    @unittest.skip
    def test_setPrint(self):
        self.fail()

    @unittest.skip
    def test_setData(self):
        self.fail()

    @unittest.skip
    def test_setDataString(self):
        self.fail()

    @unittest.skip
    def test_setModified(self):
        self.fail()

    @unittest.skip
    def test_setContant(self):
        self.fail()

    @unittest.skip
    def test_setComment(self):
        self.fail()

    @unittest.skip
    def test_setError(self):
        self.fail()

    @unittest.skip
    def test_toDict(self):
        self.fail()


if __name__ == '__main__':
    unittest.main()
