/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 10131 $
 * $Date: 2021-03-12 11:15:10 +0100 (Fri, 12 Mar 2021) $
 */
package at.lcm.symspace.rest;

import java.awt.Desktop;
import java.io.IOException;
import java.net.ServerSocket;
import java.net.URI;

import org.glassfish.grizzly.http.server.HttpServer;
import org.glassfish.jersey.grizzly2.httpserver.GrizzlyHttpServerFactory;
import org.glassfish.jersey.server.ResourceConfig;

import at.lcm.bu21.magopt.structure.Project;
import at.lcm.bu21.magopt.utils.MagOptConfig;
import at.lcm.symspace.optimizer.rest.plot.CorsFilter;

/**
 * Main class.
 *
 */
public class RESTServer {

	private static int defaultPort = 0;
	private static int port = 0;
	private static String protocol = "http";
	private static String host = "localhost";

	private static HttpServer server = null;

	// Base URI the Grizzly HTTP server will listen on
	public static String getBaseUri() {
//		return "https://localhost:" + port + "/symspace/";
		return protocol + "://" + host + ":" + port + "/symspace";
	}

	public static void setDefaultPort(int port) {
		defaultPort = port;
	}

	public static int getPort() {
		return port;
	}

	public static void browse() {
		if (getProject() != null && isAvailable()) {
			if (!isRunning()) {
				startServer();
			}

			try {
				URI uri = new URI(String.format("%s/#/project/remote/%s/%s/%d", MagOptConfig.getConfig("WebguiURL"),
						protocol, host, getPort()));
				System.out.println("browse: " + uri);
				if (Desktop.isDesktopSupported()) {
					Desktop.getDesktop().browse(uri);
				}
			} catch (Exception e) {
				System.err.println(e.getMessage());
				e.printStackTrace();
			}
		}
	}

	/**
	 * Starts Grizzly HTTP server exposing JAX-RS resources defined in this
	 * application.
	 * 
	 * @return Grizzly HTTP server.
	 */
	public static void startServer() {

		stopServer();

		try {
			ServerSocket ss = new ServerSocket(defaultPort);
			port = ss.getLocalPort();
			ss.close();

			// create a resource config that scans for JAX-RS resources and providers
			// in at.lcm.symspace.rest package
			final ResourceConfig rc = new ResourceConfig();
			URI uri = new URI(MagOptConfig.getConfig("WebguiURL"));
			String host = String.format("%s://%s", uri.getScheme(), uri.getHost());
			if (uri.getPort() > 0) {
				host += ":" + uri.getPort();
			}
			rc.register(new CorsFilter(host));
			rc.register(TreeStructure.class);
			rc.register(ProjectDetails.class);
//		rc.packages("at.lcm.symspace.rest");

//		SSLContextConfigurator sslContextConfiguration = new SSLContextConfigurator();
//		sslContext.setKeyStoreFile(keyStoreFile);

			// create and start a new instance of grizzly http server
			// exposing the Jersey application at BASE_URI
//		return GrizzlyHttpServerFactory.createHttpServer(URI.create(BASE_URI), rc, true,
//				new SSLEngineConfigurator(sslContextConfiguration, false, false, false));
			server = GrizzlyHttpServerFactory.createHttpServer(URI.create(getBaseUri()), rc);

		} catch (Exception e) {
			System.err.println(e.getMessage());
			e.printStackTrace();
		}

	}

	/**
	 * Stops Grizzly HTTP server exposing JAX-RS resources defined in this
	 * application.
	 * 
	 * @return Grizzly HTTP server.
	 */
	public static void stopServer() {
		if (server != null) {
			server.shutdownNow();
			server = null;
			port = 0;
		}
	}

	public static boolean isRunning() {
		return server != null;
	}

	public static Project getProject() {
		return ProjectDetails.getProject(ProjectDetails.getProjectId());
	}

	public static void setProject(Project project) {
		ProjectDetails.setProject(project);
	}

	public static boolean isAvailable() {

		if (isRunning()) {
			return true;
		}

		try {
			new ServerSocket(port).close();
			return true;
		} catch (IOException e) {
//			System.err.println(e.getMessage());
//			e.printStackTrace();
			return false;
		}

	}

	/**
	 * Main method.
	 * 
	 * @param args
	 * @throws IOException
	 */
	public static void main(String[] args) throws IOException {

		for (String str : args) {
			System.out.println(str);
		}
		try {
			ProjectDetails.setProject(Project.load("X:\\Demo4.mop"));
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		startServer();
//		System.out.println(String.format(
//				"Jersey app started with WADL available at " + "%sapplication.wadl\nHit enter to stop it...",
//				getBaseUri()));
		System.out.println(String.format("Jersey app started at %s\nHit enter to stop it...", getBaseUri()));
		System.in.read();
		stopServer();
	}

}
